/**************************************************************************

    tefax  -- front end to handle FAXes with efax or mgetty&sendfax
              and certain programms for view and print

    Copyright (C) 1997  Kai Uwe Tempel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    please send any bug reports, suggestions, greetings etc. to
	temu@server3.medizin.uni-leipzig.de

    or contact me in
	Kai Uwe Tempel, Gaertnerstr. 171, 04209 Leipzig, Germany

    call or fax: +49 341 4126014

***************************************************************************/
/****************************************************************************

tefax scan dir funktions

*****************************************************************************

diese datei benoetigt folgende externe funktionen

isg3, isefaxmail, lastchar, DSC

time, system, strings, mem, file

****************************************************************************/

/*LINTLIBRARY */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <dirent.h>
#include <stdlib.h>
#include <sys/stat.h>	
#include <time.h>
#include <curses.h>
#include "debug.h"
#include "tefaxdef.h"
#include "hsp.h"
#include "hcs.h"
#include "hlf.h"
#include "his.h"
#include "hul.h"
#include "hnc.h"

/***************************************************************************/

/* funktion zum einlesen der telefonbuchdatei */

int scanfaxdir(const char *server, const char *faxdir, struct unilist *flist, const char *fdformat);



/* 
** bei dateien in der lokalen verzeichnisstruktur kann das eingangsdatum
** durch einen aufruf von stat() festgestellt werden
*/

char *filedate(const char *s, struct unilist *unode)
{
  struct stat stbuf;
  struct tm *tp;
  static char d[11];

  CF(filedate);
  strcpy(d, "none");

  if (stat(s, &stbuf) == 0) {
    tp = localtime(&stbuf.st_mtime);
    unode->day = tp->tm_mday;
    unode->month = tp->tm_mon + 1;
    unode->year = tp->tm_year + 1900;
  }

  CE;
  return d;
}

/* 
** da die angaben in den dateinamen er fax- oder hiflsdateien meistens
** nur die faxnummer des absenders enthalten, muss die liste p mit den
** faxnummern auf eine passende sequenz durchsucht werden. s enthaelt
** im feld desc den suchstring, der mindestens 4 zeichen enthalten sollte,
** um fehler zum beispiel durch dreistellige hausnummern, die identisch
** mit den letzten 3 stellen von anderen faxnummern sind auszuschliessen.
** das problem liegt im vergleich vom stringende her, um faxnummern im
** ortsbereich trotz der angabe der vorwahl im absender zu erkennen.
** die funktion brich ab, wenn ein zeichen (nicht 1-9 space minus)
** erkannt wird oder der anfang eines (such- oder vergleichs-)string
** erreicht wird
*/

struct unilist *number_to_name(struct unilist *s, struct unilist *p)
{
  char *p1, *s1; 
  int i = 0;
  
  CF(number_to_name);
  if (!s || !p) { CE; return NULL; }
  if (!s->desc) { CE; return NULL; }
  if (strlen(s->desc) < 4) { CE; return NULL; }
  for (p = p->next; p != NULL; p = p->next) {
    /* vergleich vom ende her */
    p1 = p->desc + strlen(p->desc);
    s1 = s->desc + strlen(s->desc);
    while (p1 > p->desc && s1 > s->desc) {
      i = 1;
      if (*p1 != *s1) { /* leerzeichen und '-' ausschalten */
	while((*p1 == '-' || isspace(*p1)) && p1 > p->desc) {p1--; i = 0;} 
	while((*s1 == '-' || isspace(*s1)) && s1 > s->desc) {s1--; i = 0;} 
	if (i) break; /* kein gueltiges zeichen mehr */
      }
      else
	{p1--; s1--;}
    }
    /* mindestens ein string vollstaendig */
    if (p1 == p->desc || s1 == s->desc) { CE; return p; }
    if (strcmp(p->desc, s->name) == 0) { CE; return p; }
  }
  CE;
  return NULL;
}

/* 
** einfach eine listenfunktion: passenden knoten finden 
*/

char *name_to_number(const char *s, struct unilist *p)
{
  CF(name_to_number);
  for(p = p->next; p != NULL; p = p->next)
    if (strcmp(s, p->name) == 0) { CE; return p->desc; }
  CE;
  return NULL;
}


/* 
** liefert unsortierte einfach verkettete liste mit den faxen 
*/

int striptofaxlist(struct unilist *root, char *server, char *faxdir, char *faxdir_fmt)
{
  struct unilist *flist;
  struct unilist *nlist = getuninode();
  struct unilist *faxlist = getuninode();
  struct unilist *faxnode, *namenode;
  char *s = NULL;
  
  CF(striptofaxlist);
  
  /* telefonbuchdatei einlesen */
  scanfaxdir(server, faxdir, nlist, faxdir_fmt);

  /* reine faxdateien in der dateiliste des eingangesverzeichnisses
  ** selektieren, das vereinfacht den aufbau der faxliste */
  uniselect(root, select_g3);

  /* aufbau faxliste aus den g3-files. mehrfach auftretende namen werden
  ** zu einem fax mit mehreren seiten zusammengefasst */
  for (flist = root->nextsel; flist != NULL; flist = flist->nextsel) {
    *strrchr(flist->name, '.') = '\0';
    if (*flist->name == 'f' && ((s = strchr(flist->name, '-')))) *s++ = '\0';
    else s = NULL;

    for (faxnode = faxlist->next; faxnode != NULL; faxnode = faxnode->next)
      if (strcmp(flist->name, faxnode->name) == 0) { faxnode->cnt++; break; }
    if (faxnode == NULL)
      insuninode(faxlist, flist->name, NULL, s, 1, (flist->name[1] == 'f') ? 'F' : 'N', 
		 flist->day, flist->month, flist->year);
  }
  /* daten aus efaxmails */
  for (flist = root->next; flist != NULL; flist = flist->next) {
    if (isefaxmail(flist->name)) {
      strcpy(flist->name, flist->name);
      if ((s = strchr(flist->name, '.'))) *s++ = '\0';
      else s = NULL;
      for (faxnode = faxlist->next; faxnode != NULL; faxnode = faxnode->next)
	if (strcmp(flist->name + 3, faxnode->name) == 0) { 
	  free(faxnode->desc); faxnode->desc = NULL;
	  if (s) faxnode->desc = strdup(s);
	  faxnode->res = (flist->name[1] == 'f') ? 'F' : 'N';
	  break;
	}
    }
  }
  /* absender finden */
  for (faxnode = faxlist->next; faxnode != NULL; faxnode = faxnode->next) {
    if ((namenode = number_to_name(faxnode, nlist))) {
      free(faxnode->desc); faxnode->desc = NULL;
      free(faxnode->full); faxnode->full = NULL;
      faxnode->desc = strdup(namenode->name);
      faxnode->full = strdup(namenode->desc);
    }
    if (!faxnode->desc) faxnode->desc = strdup(S_(msg_unknown));
  }

  /* wurzel der dateiliste an die faxliste vererben */
  faxlist->name = root->name; root->name = NULL;
  faxlist->full = root->full; root->full = NULL;
  faxlist->desc = root->desc; root->desc = NULL;
  faxlist->day = root->day;

  freeunilist(nlist); nlist = NULL;
  moveuniroot(faxlist, root);
  freeunilist(faxlist); faxlist = NULL;
  CE;
  return root->res;
}

/* 
** zur feststellung der eingangsdatums aus den angaben von ls -l muss bei
** dateien, die juenger als ein halbes jahr sind die jahreszahl aus dem
** aktuellen datum abgeleitet werden, weil das feld fuer das jahr die tages-
** zeit enthaelt.
*/

int ayear(const int d, const int m)
{
  struct tm *tp;
  time_t tt;

  CF(ayear);
  CE;
  if (time(&tt) > 0) tp = localtime(&tt);
  else return 3000;

  if (tp->tm_mon < m) return tp->tm_year - 1 + 1900;
  if (tp->tm_mon > m) return tp->tm_year + 1900;
  if (tp->tm_mday < d) return tp->tm_year - 1 + 1900;
  else return tp->tm_year + 1900;
}

int scanlocaldir(const char *dir, struct unilist *flist)
{
  struct dirent *dp;
  DIR *dfd;
  struct stat stbuf;
  char namebuf[BUFSIZ];
  int len;
  struct unilist *dnode;
  
  CF(scanlocaldir);
 
  strcpy(namebuf, dir);
  len = strlen(namebuf);

  if ((dfd = opendir(dir)) == NULL) { CE; return -1; }
/**************** fuer jeden eintrag im directory ******************/

  while((dp = readdir(dfd)) != NULL) {
    if (strcmp(dp->d_name, ".") == 0) continue; 
    if (strcmp(dp->d_name, "..") == 0) continue;
    strcpy(namebuf + len, dp->d_name);
/**************** fehlermeldung *************************************/

    if (stat(namebuf, &stbuf) == -1) continue;

/**************** case directory ************************************/
	
    if S_ISDIR(stbuf.st_mode)
		strcat(namebuf,"/");
    
/**************** passende eintraege in die liste ********************/
/*       S(scanlocaldir, namebuf); */
/*       I(integer, len);          */
/*       P(pointer, namebuf);      */
    if ((dnode = insuninode(flist, namebuf + len, namebuf, NULL, 0, 0, 0, 0, 0)))
      filedate(namebuf, dnode);
  }
  CE;
  return flist->cnt;
} 

int scanserverdir(const char *server, const char *dir, struct unilist *flist)
{
  FILE *ffs;			/* fax from server */
  int len, pathlen;
  struct unilist *dnode;
  char line[BUFSIZ];
  char namebuf[BUFSIZ] = "";
  char fullbuf[BUFSIZ] = "";
  char comline[BUFSIZ];
  int datum;
  char monat[5] = "";
  int year;

  CF(scanserverdir);

  if (!dir || !flist || !server) {CE; return -1;}
  strcpy(fullbuf, dir);
  pathlen = strlen(fullbuf);

  /* auf dem faxserver das ergebnis von ls -l in eine datei schreiben */
  sprintf(comline, "%s %s %s %s %s \">\" ~/.faxatserver 2>> %s", 
	  S_(cl_rshell), server, S_(cl_ls), S_(cl_lso), dir, S_(fl_logfile));
  if (shellxx(comline) != 0) {CE; return -1;}

  /* die datei auf den lokalen rechner kopieren */
  sprintf(comline, "%s %s:.faxatserver %s 2>> %s", 
	  S_(cl_rcopy), server, S_(fl_faxatserver), S_(fl_logfile));
  if (shellxx(comline) != 0) {CE; return -1;}

  /* und auslesen */
  if ((ffs = fopen(S_(fl_faxatserver), "r")) == NULL) {CE; return -1;}
  while ((len = getline(line, BUFSIZ, ffs)) > 0) {
  /* die zeile enthaelt entweder die zeit des letzten schreibzugriffs hh:mm ---- */
    if (sscanf(line, "%*s %*d %*s %*s %*s %s %d %*d:%*d %[^\n]",
	       monat, &datum, namebuf) == 3) { 
      strcpy(fullbuf + pathlen, namebuf);
      if ((dnode = insuninode(flist, namebuf, fullbuf, NULL, 0, 0, datum, 0, 0))) {
	dnode->month = monthtoi(monat);
	dnode->year = ayear(datum, dnode->month);
      }
    }
  /* ---- oder das jahr */
    else if (sscanf(line, "%*s %*d %*s %*s %*s %s %d %d %[^\n]", 
		      monat, &datum, &year, namebuf) == 4) { 
      strcpy(fullbuf + pathlen, namebuf);
      if ((dnode = insuninode(flist, namebuf, fullbuf, NULL, 0, 0, datum, 0, 0))) {
	dnode->month = monthtoi(monat);
	dnode->year = ayear(datum, dnode->month);
      }
    }
  }
  fclose(ffs);
  CE;
  return flist->cnt;
}

/* telefonbuchdatei einlesen */

int scanfaxdir(const char *server, const char *faxdir, struct unilist *flist, const char *fdformat)
{
  FILE *ffs;			/* fax from server */
  int len;
  char line[BUFSIZ];
  char namebuf[BUFSIZ] = "";
  char numbuf[BUFSIZ] = "";
  char comline[BUFSIZ];

  CF(scanfaxdir);
  

  /* das telefonbuch kopieren. dadurch wird nie die originaldatei geaendert, egal welche
  ** quelle als faxverzeichnis (lokal oder remote) dient */
  if (server) sprintf(comline, "%s %s:%s %s 2>> %s", 
		      S_(cl_rcopy), server, S_(fl_faxdirfile), S_(fl_faxdircopy), S_(fl_logfile));
  else sprintf(comline, "%s %s %s 2>> %s", 
	       S_(cl_copy), S_(fl_faxdirfile), S_(fl_faxdircopy), S_(fl_logfile));
  if (shellxx(comline) != 0) {CE; return -1;}
  if ((ffs = fopen(S_(fl_faxdircopy), "r")) == NULL) {CE; return -1;}
 
  /* durch die variable fdformat aus der konfigurationsdatei kann mit den folgenden drei
  ** zeilen im prinzip jedes format der telefonbuchdatei gelesen werden */
  while ((len = getline(line, BUFSIZ, ffs)) >= 0) {
    if (sscanf(line, fdformat, namebuf, numbuf) == 2) {
      insuninode(flist, namebuf, NULL, numbuf, 0,0,0,0,0);
    }
  }
  fclose(ffs);

  CE;
  return flist->cnt;
}


/* 
** da bei jeder empfaengerauswahl die faxnummer editiert werden kann, muss die
** namensliste zurueckgeschrieben werden
*/
int updatefaxdir(const char *server, const char *faxdir, struct unilist *flist, 
		 const char *fdformat, const char *fwformat)
{
  FILE *ffs;			/* fax from server */
  FILE *ffu;			/* update */
  int len;
  char line[BUFSIZ];
  char namebuf[BUFSIZ] = "";
  char numbuf[BUFSIZ] = "";
  char comline[BUFSIZ];
  int insert = 1;

  CF(updatefaxdir);
  pacline(line_st, S_(sts_ud));

  if (!flist->desc) insert = 0;

  /* sicher die aktuelle version bearbeiten */
  if (server) sprintf(comline, "%s %s:%s %s", S_(cl_rcopy), server, S_(fl_faxdirfile), S_(fl_faxdircopy));
  else sprintf(comline, "%s %s %s", S_(cl_copy), S_(fl_faxdirfile), S_(fl_faxdircopy));
  if (shellxx(comline) != 0) {CE; return -1;}

  if ((ffs = fopen(S_(fl_faxdircopy), "r")) == NULL) {CE; return -1;}
  if ((ffu = fopen(S_(fl_faxdirtemp), "w")) == NULL) {CE; return -1;}
  
  while ((len = getline(line, BUFSIZ, ffs)) >= 0) {
    /* ggf. eintrag aendern */
    if ((sscanf(line, fdformat, namebuf, numbuf) == 2) &&
	(strcmp(flist->name, namebuf) == 0)) {
	if (insert) fprintf(ffu, fwformat, flist->name, flist->desc);
	insert = 0;
    }
    /* oder eintrag uebernehmen */
    else fprintf(ffu, "%s\n", line);
  }
  /* oder hinzufuegen */
  if (insert) fprintf(ffu, fwformat, flist->name, flist->desc);

  fflush(ffu);
  fclose(ffs);
  fclose(ffu);

  /* zum schluss die neue datei an die alte stelle kopieren */
  if (server) sprintf(comline, "%s %s %s:%s", S_(cl_rcopy), S_(fl_faxdirtemp), server, S_(fl_faxdirfile));
  else sprintf(comline, "%s %s %s", S_(cl_copy), S_(fl_faxdirtemp), S_(fl_faxdirfile));
  if (shellxx(comline) != 0) {CE; return -1;}
  
  
  CE;
  return insert;
}

/* 
** getdire() laesst eine liste mit den eintraegen eines beliebigen verzeichnisses
** bauen. das einzulesende verzeichnis kann auch im wurzelknoten der dateiliste
** flist uebegeben werden. enthaelt server einen NULL-zeiger, wird das verzeichnis
** lokal eingelesen 
*/
char *getdire(const char *server, const char *dir, struct unilist *flist)
{
  static char pfad[BUFSIZ];

  CF(getdire);
				/* reset */
  pfad[0] = '\0';
  if (flist->next)
    freeunilist(flist->next);
  flist->next = NULL;
  flist->nextsel = NULL;
  flist->cnt = 0;
  flist->res = 0;
				/* klare verhaeltnisse */
  if (dir) strcpy(pfad, dir);
  else if (flist->full)  strcpy(pfad, flist->full);
  if (*pfad != '/')  strcpy(pfad, "/");
  if (lastchar(pfad) != '/') *(strrchr(pfad, '/') + 1) = '\0'; 
				/* scan! */

  if (server) scanserverdir(server, pfad, flist);
  else scanlocaldir(pfad, flist);

  CE;
  return pfad;
}

















