/**************************************************************************

    tefax  -- front end to handle FAXes with efax or mgetty&sendfax
              and certain programms for view and print

    Copyright (C) 1997  Kai Uwe Tempel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    please send any bug reports, suggestions, greetings etc. to
	temu@server3.medizin.uni-leipzig.de

    or contact me in
	Kai Uwe Tempel, Gaertnerstr. 171, 04209 Leipzig, Germany

    call or fax: +49 341 4126014

***************************************************************************/
/***************************************************************************

tefax scan dir haeder file

***************************************************************************/
/***************************************************************************

sd stellt folgende funktionen zur verfuegung

zum aufbau von unilisten

***************************************************************************/

/* NULL bei fehler */
struct unilist *getuninode(void);

/* NULL bei fehler, belegt alle member mit den uebergebenen werten
   und incrementiert root->cnt, liefert zeiger auf eingefuegten
   knoten, das ist root->next, bei nextsel-funktionen nicht selektiert*/
struct unilist *insuninode(struct unilist *root,
		  char *name, char *full, char *desc,
		  int cnt, int res, int day, int month, int year);

/* setzt alle zeiger die auf delnode zeigen auf delnode->next bzw.
   delnode->nextsel, und loest den knoten mit freeuninode() auf.
   gibt in root->nextsel einen zeiger zurueck in folgender prioritaet:
   delnode->nextsel, delnode->next, uniselect_all, NULL.
   nichts, falls root oder delnode NULL*/
void remuninode(struct unilist *root, struct unilist *delnode);

/* gibt alle (char *) frei, zuletzt unode
   nichts, falls unode NULL*/
void freeuninode(struct unilist *unode);

/* loest alle knoten mit freeuninode auf */
void freeunilist(struct unilist *p);

/* laesst nur noch einen leeren knoten uebrig
   nichts, falls p NULL*/
void cleanunilist(struct unilist *p);

/* gibt alle (char *) frei, alle int = 0,
   nichts, falls p NULL*/
void cleanuninode(struct unilist *p);

/* vertauscht root und newroot wenn beide nicht NULL */
void moveuniroot(struct unilist *root, struct unilist *newroot);

/* haengt alle selektierten eintraege aus or an die doppelt verkettete
   liste nr an. die knoten werden nicht kopiert, sondern aus or entfernt.
   nr->nextsel wird nicht veraendert, es sei denn nextsel ist null, dann
   wird nextsel = next gesetzt
   or wird anschliessend mit selectall neu verkettet: or->res */
int movenodes(struct unilist *or, struct unilist *nr, int (*select)(const struct unilist *, const struct unilist *));

/***************************************************************************

zum sortieren der unilist 

***************************************************************************/

/* unisort verkettet die liste (auch leere) vorwaerts (next) neu 
   nichts wenn root NULL*/
void unisort(struct unilist *root, int (*compare)(const void *, const void *));

/* selsort ruft nach unisort() noch doublechain() auf, damit die liste wieder 
   voll doppelt verkettet ist */
void selsort(struct unilist *root, int (*compare)(const void *, const void *));

/* vergleichsfunktionen */
int compname(const void *p, const void *q);
int compdate(const void *p, const void *q);
int compdesc(const void *p, const void *q);

/* diese funktionen sind kopien der entsprechenden originale, aber durch das 
   vertauschen der eingabeparameter ist das ergebnis negiert */
int comp_name(const void *p, const void *q);
int comp_date(const void *p, const void *q);
int comp_desc(const void *p, const void *q);


/***************************************************************************

zur modifikation

***************************************************************************/

/* add-extension: fuer g3 an faxspool entwickelt, wird an alle g3-files die
** extension '.g3' angehaengt, damit faxspool das format dieser dateien er-
** kennen kann. diese namen werden fuer links im ausgangsverzeichnis auf
** die originaldateien verwendet. erweitert werden nur die eintraege in 
** ->name */
void addex(struct unilist *root);

/***************************************************************************

zum selektieren in der unilist 

***************************************************************************/

/* alle nextsel-zeiger zeigen in next-richtung auf den letzten selektierten
   eintrag, nichts wenn root NULL */
int uniselect(struct unilist *root, 
	      int (*select)(const struct unilist *, const struct unilist *));

struct unilist *selselect(struct unilist *root);

/* fuer alle eintraege isg3(p->name) return 1 */
int select_g3(const struct unilist *p, const struct unilist *q);

/* return immer 1 */
int select_all(const struct unilist *p, const struct unilist *q);

/* eintraege werden bis zur laenge p->name verglichen, return 1 bei gleichheit */
int select_tor(const struct unilist *p, const struct unilist *q);

/* 1 nur bei gleichheit p->name */
int select_one(const struct unilist *p, const struct unilist *q);

/* wie _g3 aber die mailfiles werden auch erfasst */
int select_fax(const struct unilist *p, const struct unilist *q);

/* nur 1 wenn letztes zeichen p->name kein slash */
int select_nod(const struct unilist *p, const struct unilist *q);

/* findsel setzt root-name root-full und root-desc mit strdup auf die werte 
   des sel. eintrags: entweder nach gleichheit -name oder der n-te eintrag */
void findsel(struct unilist *root);


/* countsel zaehlt root->res und root->day neu aus. das ergigt wieder einen 
   konsistenten zustand */
void countsel(struct unilist *root);

/* doublechain verkettet die liste rueckwaerts wie select_all, veraendert aber
   den zeiger root->nextsel nicht */
int doublechain(struct unilist *root);

/* findmaxeq findet den laengsten, mit strncmp auf root->name passenden string
   unter allen selektierten eintraegen */
char *findmaxeq(struct unilist *f);

/***************************************************************************

zum ausgeben der unilist 

***************************************************************************/

/* diese funktionen schreiben die daten von f in einen static string */
char *pn_line(struct unilist *f);
char *pn_fax(struct unilist *f);
char *pn_name(struct unilist *f);
char *pn_file(struct unilist *f);
char *pn_full(struct unilist *f);


/***************************************************************************

zum testen der unilist 

***************************************************************************/

void test(struct unilist *f);
void testsel(struct unilist *f);
void testfax(struct unilist *f);
void checkul(struct unilist *node, char *mess);






