/**************************************************************************

    tefax  -- front end to handle FAXes with efax or mgetty&sendfax
              and certain programms for view and print

    Copyright (C) 1997  Kai Uwe Tempel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    please send any bug reports, suggestions, greetings etc. to
	temu@server3.medizin.uni-leipzig.de

    or contact me in
	Kai Uwe Tempel, Gaertnerstr. 171, 04209 Leipzig, Germany

    call or fax: +49 341 4126014

***************************************************************************/
/****************************************************************************

tefax main macro function file

***************************************************************************/

/*LINTLIBRARY */

#include <stdio.h>
#include <curses.h>
#include "tefaxdef.h"
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include "debug.h"
#include "hdw.h"
#include "hul.h"
#include "hcs.h"
#include "hnc.h"
#include "hrc.h"
#include "hsp.h"
#include "hlf.h"
#include "his.h"

/******************************************************************************
******************************************************************************/

/* 
** only for development 
*/
void mm_dummi(void)
{}


/* die uniliste enthaelt die dateinamen, sowie in der wurzel den empfaenger (name) und dessen
** faxnummer (desc). 
*/
void mm_send(char *server, struct unilist *p)
{
  char *pcl = NULL;		/* pointer comline */
  char *pfl = NULL;		/* pointer filelist */
  char sres[20];
  int failure = 0;

  if (!p || !p->desc || !p->next) return;
  CF(mm_send);

  /* kommandozeilenschalter fuer die aufloesung einstellen und ggf. durch ein leerzeichen
  ** von den weiteren parametern trennen */
  strcpy(sres, (O_GET(ARG_L)) ? S_(co_low) : S_(co_fin));
  if (strlen(sres)) strcat(sres, " ");

  /* wenn ein remote host die faxe verschickt ist co_server nicht NULL */
  if (S_(co_server)) {

    /* wenn server nicht NULL ist, befinden sich die dateien auf dem entfernten rechner
    ** diese option wird bei forward verwendet, um die dateien nicht noch einmal uebers
    ** netz kopieren zu muessen */
    if (server) {

      /* faxspool erkennt in der derzeigen version das dateiformat an der extension.
      ** daher weren alle dateien mit einem erweiterten link im outgoing_path versehen
      */
      addex(p);
      if ((pfl = print_fll(S_(pth_rog), p))) {
	pcl = (char *) malloc(strlen(S_(cl_rshell)) + strlen(server) + strlen(pfl) + 
			      strlen(S_(fl_logfile)) + 100);
	sprintf(pcl, "%s %s '%s' >> %s", S_(cl_rshell), server, pfl, S_(fl_logfile)); 
	failure = shellex(pcl);
	free(pfl); pfl = NULL; free(pcl); pcl = NULL;
      }
      else failure = 1;

      /* die dateinamen werden incl. pfadnamen aus der uniliste uebernommen und der rsh 
      ** uebergeben. die hochkommata verhindern die auswertung von leerzeichen einmal durch
      ** die shell und einmal durch die rsh */
      if (failure == 0 && (pfl = print_fil(S_(pth_rog), p))) {
	pcl = (char *) malloc(2 * strlen(S_(cl_rshell)) + 2 * strlen(server) + strlen(S_(co_send)) + 
			      strlen(p->desc) + 2 * strlen(pfl) + 2 * strlen(S_(fl_logfile)) + 100);
	sprintf(pcl, "%s %s %s %s'\"%s\"' '%s' >> %s 2>&1; %s %s %s '%s' >> %s &", 
		S_(cl_rshell), server, S_(co_send), sres,
		p->desc, pfl, S_(fl_logfile),
		S_(cl_rshell), server, S_(cl_remove), pfl, S_(fl_logfile));
	failure = shellex(pcl);
      }
      else failure = 1;
    }
    else {
      /* die angabe des benutzernamens ist in der man-page von rcp beschrieben. ich habe 
      ** die funktion nicht testen koennen */
      char bufs[BUFSIZ] = "";
      
      if (S_(co_login)) {
#ifdef SERVER_IS_BSD
	sprintf(bufs, "%s.%s:%s", S_(co_server), S_(co_login), S_(pth_rog));
#else
	sprintf(bufs, "%s@%s:%s", S_(co_login), S_(co_server), S_(pth_rog));
#endif
      }

      /* (co_server) && (server == NULL) 
      ** -----> dateien befinden sich lokal und werden remote auf dem faxserver
      ** verschickt
      */
      else sprintf(bufs, "%s:%s", S_(co_server), S_(pth_rog));

      /* faxspool erkennt in der derzeigen version das dateiformat an der extension.
      ** daher weren alle dateien mit einem erweiterten link im outgoing_path versehen
      */
      addex(p);
      if ((pfl = print_fll(S_(pth_og), p))) {
	pcl = (char *) malloc(strlen(pfl) + strlen(S_(fl_logfile)) + 100);
	sprintf(pcl, "%s >> %s", pfl, S_(fl_logfile)); 
	failure = shellex(pcl);
	free(pfl); pfl = NULL; free(pcl); pcl = NULL;
      }
      else failure = 1;

      /* die zu sendenden dateien befinden sich auf dem lokalen rechner, wenn server auf
      ** NULL zeigt, und muessen zuerst auf den remote host kopiert werden. dazu werden 
      ** die dateinamen incl. pfad an rcp uebergeben. die dateien landen auf dem server im
      ** verzeichnis rog. dieses verzeichnis wird bei der zweiten auflistung (fuer die rsh)
      ** als quellverzeichnis verwendet. die hochkommata verhindern die auswertung von
      ** leerzeichen einmal durch die shell und einmal durch die rsh */
      if (failure == 0 && (pfl = print_fil(S_(pth_og), p))) {
	pcl = (char *) malloc(strlen(S_(cl_rcopy)) + strlen(pfl) + strlen(bufs) +
			      strlen(S_(fl_logfile)) + 100);
	sprintf(pcl, "%s %s %s >> %s 2>&1", 
		S_(cl_rcopy), pfl, bufs, S_(fl_logfile));
	pacline(line_st, S_(sts_uf));
	failure = shellex(pcl);
	free(pfl); pfl = NULL; free(pcl); pcl = NULL;
      }
      else failure = 1;
      
      if ((failure == 0) && (pfl = print_fil(S_(pth_rog), p))) {
	pcl = (char *) malloc(strlen(S_(cl_rshell)) + strlen(S_(co_server)) + strlen(S_(co_send)) + 
			      strlen(p->desc) + strlen(pfl) + strlen(S_(fl_logfile)) + 100);
	sprintf(pcl, "%s %s %s %s'\"%s\"' '%s' >> %s 2>&1 &", 
		S_(cl_rshell), S_(co_server), S_(co_send), sres,
		p->desc, pfl, S_(fl_logfile));
	failure = shellex(pcl);
	free(pfl); pfl = NULL; free(pcl); pcl = NULL;
      }
      else failure = 1;

      if ((pfl = print_fil(S_(pth_og), p))) {
	pcl = (char *) malloc(strlen(pfl) + strlen(S_(fl_logfile)) + 100);
	sprintf(pcl, "%s %s >> %s", S_(cl_remove), pfl, S_(fl_logfile)); 
	shellex(pcl);
	free(pfl); pfl = NULL; free(pcl); pcl = NULL;
      }
    }
  }
  /* wenn das faxprogramm auf dem lokalen host laeuft (co_server = NULL) werden die dateinamen 
  ** aus der uniliste incl. pfad, sowie faxnummer und der schalter fuer die aufloesung an das 
  ** faxprogramm uebergeben. die hochkommata verhindern die auswertung von leerzeichen in der 
  ** faxnummer durch die shell */
  else {

    /* faxspool erkennt in der derzeigen version das dateiformat an der extension.
    ** daher weren alle dateien mit einem erweiterten link im outgoing_path versehen
    */
    addex(p);
    if ((pfl = print_fll(S_(pth_og), p))) {
      pcl = (char *) malloc(strlen(pfl) + strlen(S_(fl_logfile)) + 100);
      sprintf(pcl, "%s >> %s", pfl, S_(fl_logfile)); 
      failure = shellex(pcl);
      free(pfl); pfl = NULL; free(pcl); pcl = NULL;
    }
    else failure = 1;
    /* bei der uebergabe an das faxprogramm werden nur die links auf die originaldateien
    ** in pth_og verwendet. die links werden geloescht, nachdem das faxprogramm seine
    ** arbeit beendet hat
    */
    if (failure == 0 && (pfl = print_fil(S_(pth_og), p))) {
      pcl = (char *) malloc(strlen(S_(co_send)) + strlen(p->desc) + 
				   2 * strlen(pfl) + strlen(S_(fl_logfile)) + 100);

      sprintf(pcl, "%s %s\"%s\" %s >> %s 2>&1; %s %s >> %s &", 
	      S_(co_send), sres, p->desc, pfl, S_(fl_logfile),
	      S_(cl_remove), pfl, S_(fl_logfile));

      failure = shellex(pcl);
    }
    else failure = 1;
  }

  free(pfl); pfl = NULL; free(pcl); pcl = NULL;

  if (failure) pacline(line_st, S_(sts_nos));
  else pacline(line_st, S_(sts_sent));

  CE;
}


/* invocingeditor() generiert einen unique dateinamen, aus dem empfaengernamen in der wurzel von 
** forsend, ruft einen editor auf und traegt nach
** dem speichern den dateinamen in die uniliste ein. wird die datei vor dem verlassen des editors
** nicht gespeichert, bleibt die uniliste unveraendert und die funktion gibt null zurueck 
** andernfalls eine eins 
*/
int invocingeditor(struct unilist *forsend)
{
  char *pfad = S_(pth_og);
  char nameff[BUFSIZ] = "";
  char name[BUFSIZ] = "";
  char *s, *c;
  int i = 0;
  char t[50] = "";
  struct tm *tp;
  time_t tt;

  if (!forsend->name) return 0;
  CF(invoceditor);

  strcpy(nameff, forsend->name);
  c = nameff;
  s = name;
  /* leerzeichen aus dem empfaengernamen entfernen */
  while (!isalnum(*c) && *c != '\0') c++; /* vorher */
  while((*s++ = *c++) != '\0')
    while (!isalnum(*c) && *c != '\0') c++;	/* und den rest */

  if (time(&tt) > 0) {		/* die zeit fuer einen unique_name */
    tp = localtime(&tt);
    strftime(t, 10, "%d%m.%H%M", tp);
  }
  else strcpy(t, "TEMP");	/* notfalls temp */

  sprintf(nameff, "%sfax-%s.%s.txt", pfad, cutonend(name, 10), t);

  /* der editor ist abhaengig von der konfiguration */
  if (O_GET(ARG_x)) {
    if (O_GET(avl_xedit))
      sprintf(name, "%s %s", S_(avl_xedit), nameff);
    else 
      sprintf(name, "%s %s %s", S_(co_x_edit), S_(avl_edit), nameff);
  }
  else sprintf(name, "%s %s", S_(avl_edit), nameff);
  i = shellex(name);
  
  /* umwandlung des rueckgabewertes: fehlerfreie beendigung der shell mit return null -> eins
  ** und eintrag der datei in die uniliste. sonst return null */
  if (i++) i = 0;
  else if (isfile(nameff) == 1)
	forsend->nextsel = insuninode(forsend, nameff + strlen(pfad), 
				      nameff, NULL, 0,0,0,0,0);
  CE;
  return i;
}


/* kopieren der eingegangenen faxfiles in ein lokales verzeichnis, wenn ein faxserver
** existiert. die uniliste enthaelt die namen der angeforderten dateien 
** wo_proxy verhinder mehrfaches kopieren derselben dateien
*/
int mm_getfiles(struct unilist *root)
{
  char buf[BUFSIZ] = "";
  char bufs[BUFSIZ] = "";
  int i = 1;

  if (!root || !root->next) return 0;
  CF(mm_getfiles);

  /* die angabe des benutzernamens ist in der man-page von rcp beschrieben. ich habe 
  ** die funktion nicht testen koennen */
  if (S_(co_login)) {
#ifdef SERVER_IS_BSD
    sprintf(bufs, "%s.%s:%s", S_(co_server), S_(co_login), S_(pth_ic));
#else
    sprintf(bufs, "%s@%s:%s", S_(co_login), S_(co_server), S_(pth_ic));
#endif
  }
  else sprintf(bufs, "%s:%s", S_(co_server), S_(pth_ic));

  pacline(line_st, S_(sts_gf));

  /* die austellung einer liste mit den auf dem lokalen rechner noch nicht vorhandenen
  ** dateien erschien zu aufwendig, daher werden die benoetigten dateien mit rcp jeweils
  ** einzeln kopiert. */
  while ((root = root->next)) {
    if (root->name) {
      sprintf(buf, "%s%s", S_(pth_ric), root->name);
      if (isfile(buf) != 1 || !O_GET(wo_proxy)) {

	/* die ganze sache laesst sich auch im hintergrund erlegigen, wenn ans ende der 
	** formatanweisung ein ampersand angehaengt wird. das kann aber bei langsamen
	** netzen zu problemen fuehren, wenn auf die dateien zugegrifen wird, die noch 
	** gar nicht existieren */
	sprintf(buf, "%s %s%s %s 2>> %s", 
		S_(cl_rcopy), bufs, root->name, S_(pth_ric), S_(fl_logfile));
	if (shellex(buf) != 0) i = 0;
      }
    }
  }
  CE;
  return i;
}     


/* anzeige von faxdateien mittels g3view unter X11. ggf. werden die dateien mit 
** mm_getfiles geholt 
*/
void mm_g3x(char *server, struct unilist *root)
{
  char *pcl = NULL;		/* pointer comline */
  char *pfl = NULL;		/* pointer filelist */
  char sres[20];

  if (!root || !root->next) return;
  CF(mm_g3x);

  /* kommandozeilenschalter fuer die aufloesung einstellen und ggf. durch ein leerzeichen
  ** von den weiteren parametern trennen */
  strcpy(sres, (O_GET(ARG_L)) ? S_(co_low_v) : S_(co_fin_v));
  if (strlen(sres)) strcat(sres, " ");

  /* auf dem faxserver befinden sich die dateien immer im eingangsverzeichnis, auf dem lokalen
  ** rechner, koennen sie aber aus einem beliebigen verzeichnis gelesen werden */
  if (server) { if ((mm_getfiles(root))) pfl = print_fil(S_(pth_ric), root); }
  else pfl = print_fil(NULL, root);
  if (pfl) {
    if ((pcl = (char *) malloc(strlen(S_(avl_g3view)) + strlen(pfl) + 100))) {
      sprintf(pcl, "%s %s%s", S_(avl_g3view), sres, pfl);
      pacline(line_st, S_(sts_lv));
      shellex(pcl);
    }
  }
  free(pcl); pcl = NULL; free(pfl); pfl = NULL;
  CE;
}


/* anzeige von faxdateien mittels g3vga auf der console. ggf. werden die dateien mit 
** mm_getfiles geholt. g3vga akzeptiert keinen schalter zur einstellung der aufloesung
** fuer andere anzeigeprogramme muss der schalter analog mm_g3x eingefuegt werden
*/
void mm_g3c(char *server, struct unilist *root)
{
  char *pcl = NULL;		/* pointer comline */
  char *pfl = NULL;		/* pointer filelist */

  if (!root || !root->next) return;
  CF(mm_g3c);

  /* auf dem faxserver befinden sich die dateien immer im eingangsverzeichnis, auf dem lokalen
  ** rechner, koennen sie aber aus einem beliebigen verzeichnis gelesen werden */
  if (server) { if ((mm_getfiles(root))) pfl = print_fil(S_(pth_ric), root); }
  else pfl = print_fil(NULL, root);
  if (pfl) {
    if ((pcl = (char *) malloc(strlen(S_(avl_g3vga)) + strlen(pfl) + strlen(S_(fl_logfile)) + 100))) {
      sprintf(pcl, "%s %s > %s", S_(avl_g3vga), pfl, S_(fl_logfile));
      pacline(line_st, S_(sts_lg));
      shellex(pcl);
    }
  }
  free(pcl); pcl = NULL; free(pfl); pfl = NULL;
  CE;
}


/* umwandlung (convert) der faxdateien in das ps-format und anzeige oder 
** uebergabe an den printer-daemon (proceed). die dateien werden ebenso wie die 
** faxdateien numeriert
*/
void mm_pps(char *server, struct unilist *root, char *convert, char *proceed)
{
  char *pcl = NULL;		/* pointer comline */
  char *pfl = NULL;		/* pointer filelist */
  char psname[BUFSIZ];		/* name ps file */

  if (!root || !root->next) return;
  CF(mm_pps);

  /* wenn ein komplettes fax angezeigt wird, kann der faxname als dateiname verwendet werden,
  ** sonst wird ein temporaerer name vom system generiert */
  if (root->name) sprintf(psname, "%s%s", S_(pth_ps), root->name);
  else tmpnam(psname);
  
  /* auf dem faxserver befinden sich die dateien immer im eingangsverzeichnis, auf dem lokalen
  ** rechner, koennen sie aber aus einem beliebigen verzeichnis gelesen werden */
  if (server) { if ((mm_getfiles(root))) pfl = print_fil(S_(pth_ric), root); }
  else pfl = print_fil(NULL, root);
  if (pfl) {
    if ((pcl = (char *) malloc(strlen(convert) + strlen(S_(pth_ps)) + strlen(psname) +  
			       strlen(pfl) + strlen(S_(fl_logfile)) + 100))) {
      sprintf(pcl, "%s -n%s.%%d.ps %s 2>> %s", convert, psname, pfl, S_(fl_logfile));
      pacline(line_st, S_(sts_cf));
      shellex(pcl);
    }
  }
  free(pcl); pcl = NULL; free(pfl); pfl = NULL;
  
  /* die aufrufe von lpr und ghostview erfolgen nach dem gleichen schema */
  if (proceed == S_(cl_lpr) || proceed == S_(avl_ghost)) {
    if ((pcl = (char *) malloc(strlen(proceed) + strlen(S_(pth_ps)) + 
			       strlen(psname) + strlen(S_(fl_logfile)) + 100))) {
      sprintf(pcl, "%s %s.*.ps 1>> %s", proceed, psname, S_(fl_logfile));
      if (proceed == S_(cl_lpr)) pacline(line_st, S_(sts_pf));
      shellex(pcl);
    }
  }
  else {

    /* fuer gs muss das quit-file an die dateiliste angehaengt werden, damit gs nicht mit ^C 
    ** abgebrochen werden muss */
    if ((pcl = (char *) malloc(strlen(proceed) + strlen(S_(pth_ps)) + strlen(psname) + 
			       strlen(S_(co_gs_quit)) + strlen(S_(fl_logfile)) + 100))) {
      if (O_GET(ARG_x))
	sprintf(pcl, "%s %s.*.ps %s 1>> %s 2>&1", proceed, psname, S_(co_gs_quit), S_(fl_logfile));
      else
	sprintf(pcl, "%s %s.*.ps %s 1>> %s", proceed, psname, S_(co_gs_quit), S_(fl_logfile));
      pacline(line_st, S_(sts_ls));
      shellex(pcl);
    }
  }
    
  free(pcl); pcl = NULL;
  
  /* wenn sich der befehl auf eine auswahl von dateien bezieht, werden die ps-dateien nach
  ** abschluss der operation wieder geloescht, um bei weiteren aufrufen fehlfunktionen durch
  ** die existens von gleichnamigen dateien einer frueheren convertierung zu verhindern  */
  if (!root->name) {
    char buf[BUFSIZ];
    sprintf(buf, "%s %s.ps 2>> %s", S_(cl_remove), psname, S_(fl_logfile));
    shellxx(buf);
  }
  CE;
}


/* xv ist das einzige programm, mit dem die faxe bearbeitet werden koennen. leider ist
** danach eine convertierung der dateien von hand in das g3-format noetig, da keines
** der faxprogramme pgm-dateien akzeptiert */
void mm_pgm(char *server, struct unilist *root, char *convert, char *proceed)
{
  char *pcl = NULL;		/* pointer comline */
  char *pfl = NULL;		/* pointer filelist */
  char pgmname[BUFSIZ];		/* name ps file */

  if (!root || !root->next) return;
  CF(mm_pgm);

  /* wenn ein komplettes fax angezeigt wird, kann der faxname als dateiname verwendet werden,
  ** sonst wird ein temporaerer name vom system generiert */
  if (root->name) sprintf(pgmname, "%s%s", S_(pth_pgm), root->name);
  else tmpnam(pgmname);
  
  /* auf dem faxserver befinden sich die dateien immer im eingangsverzeichnis, auf dem lokalen
  ** rechner, koennen sie aber aus einem beliebigen verzeichnis gelesen werden */
  if (server) { if ((mm_getfiles(root))) pfl = print_fil(S_(pth_ric), root); }
  else pfl = print_fil(NULL, root);
  if (pfl) {
    if ((pcl = (char *) malloc(strlen(convert) + strlen(S_(pth_pgm)) + strlen(pgmname) +
			        strlen(pfl) + strlen(S_(fl_logfile)) + 100))) {
      sprintf(pcl, "%s -n%s.%%d.pgm %s 2>> %s", convert, pgmname, pfl, S_(fl_logfile));
      pacline(line_st, S_(sts_cf));
      shellex(pcl);
    }
  }
  free(pcl); pcl = NULL; free(pfl); pfl = NULL;
  
  if ((pcl = (char *) malloc(strlen(proceed) + strlen(S_(pth_pgm)) + strlen(pgmname) + 
			     strlen(S_(fl_logfile)) + 100))) {
    sprintf(pcl, "%s %s.*.pgm 2>> %s", 
	    proceed, pgmname, S_(fl_logfile));
    pacline(line_st, S_(sts_lx));
    shellex(pcl);
  }
    
  free(pcl); pcl = NULL;
  
  /* wenn sich der befehl auf eine auswahl von dateien bezieht, werden die pgm-dateien nach
  ** abschluss der operation wieder geloescht, um bei weiteren aufrufen fehlfunktionen durch
  ** die existens von gleichnamigen dateien einer frueheren convertierung zu verhindern  */
  if (!root->name) {
    char buf[BUFSIZ];
    sprintf(buf, "%s %s.*.pgm 2>> %s", S_(cl_remove), pgmname, S_(fl_logfile));
    shellxx(buf);
  }
  CE;
}


/* "loeschen" der dateien auf dem server oder im eingangsverzeichnis bedeutet ein 
** verschieben der dateien nach pth_del. richtig geloescht werden sie erst durch mm_kill */
void mm_delete(char *server, struct unilist *root)
{
  char *pcl = NULL;		/* pointer comline */
  char *pfl = NULL;		/* pointer filelist */
  char buf[BUFSIZ];
  
  if (!root || !root->next) return;
  CF(mm_delete);
  
  /* die angabe des benutzernamens ist in der man-page von rcp beschrieben. ich habe 
  ** die funktion nicht testen koennen */
  if (server) {
    if (S_(co_login)) {
#ifdef SERVER_IS_BSD
      sprintf(buf, "%s.%s:%s", server, S_(co_login), S_(pth_ic));
#else
      sprintf(buf, "%s@%s:%s", S_(co_login), server, S_(pth_ic));
#endif
    }
    else sprintf(buf, "%s:%s", server, S_(pth_ic));
    
    if ((pfl = print_fil(buf, root))) {
      if ((pcl = (char *) malloc(strlen(S_(cl_rcopy)) + strlen(pfl) + 
					strlen(S_(pth_del)) + strlen(S_(fl_logfile)) + 100))) {

	/* zuerst werden die dateien vom server mit rcopy in pth_del gesichert */
	sprintf(pcl, "%s %s %s 2>> %s", S_(cl_rcopy), pfl, S_(pth_del), S_(fl_logfile));
	pacline(line_st, S_(sts_gf));
	if (shellex(pcl) == 0) {
	  free(pfl); pfl = NULL; free(pcl); pcl = NULL;

	  /* das neue zusammenstellen der liste ist noetig, weil cl_rcopy in einer lokalen shell
	  ** laeuft, cl_remove aber in einer rsh, so dass die pfadanabe einmal relativ zum lokalen
	  ** rechner und einmal relativ zum wurzelverzeichnis auf dem server erfolgt */
	  if ((pfl = print_fil(NULL, root))) {
	    if ((pcl = (char *) malloc(strlen(S_(cl_rshell)) + strlen(server) + strlen(S_(cl_remove)) + 
				       strlen(pfl) + strlen(S_(fl_logfile)) + 100))) {
	      sprintf(pcl, "%s %s %s %s 2>> %s", S_(cl_rshell), server, S_(cl_remove), pfl, S_(fl_logfile));
	      pacline(line_st, S_(sts_fdel));
	      shellex(pcl);
	    }
	  }
	}
      }
    }
  }
  else {

    /* lokal wird einfach mit mv (cl_move) verschoben */
    if (!S_(co_server) && (pfl = print_fil(NULL, root)))
      if ((pcl = (char *) malloc(strlen(S_(cl_move)) + strlen(pfl) + 
				 strlen(S_(pth_del)) + strlen(S_(fl_logfile)) + 100))) {
	sprintf(pcl, "%s %s %s 2>> %s", S_(cl_move), pfl, S_(pth_del), S_(fl_logfile));
	shellex(pcl);
	pacline(line_st, S_(sts_fdel));
      }
  }
  free(pfl); pfl = NULL; free(pcl); pcl = NULL;
  CE;
}

/* die harte variante: die dateien werden mit dem remove-befehl unwiederbringlich geloescht
** receivedbox dient zur kontrolle, dass auch wirklich nur dateien aus dem ablageverzeichnis
** geloescht und nicht aus der eingangsbox werden
*/
void mm_kill(char *receivedbox, struct unilist *root)
{
  char *pcl = NULL;		/* pointer comline */
  char *pfl = NULL;		/* pointer filelist */
  
  if (!root || !root->next) return;
  CF(mm_kill);

  /* kontrolle des verzeichnisses. damit werden nur dateien geloescht,
  ** die sich auch wirklich im received path befnden */
  if (receivedbox == S_(pth_del) && (pfl = print_fil(NULL, root)))
    if ((pcl = (char *) malloc(strlen(S_(cl_remove)) + strlen(pfl) + 
			       strlen(S_(fl_logfile)) + 100))) {
      sprintf(pcl, "%s -f %s 2>> %s", S_(cl_remove), pfl, S_(fl_logfile));
      shellex(pcl);
      pacline(line_st, S_(sts_fdel));
    }

  free(pfl); pfl = NULL; free(pcl); pcl = NULL;
  CE;
}










