/**************************************************************************

    tefax  -- front end to handle FAXes with efax or mgetty&sendfax
              and certain programms for view and print

    Copyright (C) 1997  Kai Uwe Tempel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    please send any bug reports, suggestions, greetings etc. to
	temu@server3.medizin.uni-leipzig.de

    or contact me in
	Kai Uwe Tempel, Gaertnerstr. 171, 04209 Leipzig, Germany

    call or fax: +49 341 4126014

***************************************************************************/
/***************************************************************************
  
  tefax ncurses funktions
  
***************************************************************************/
/***************************************************************************
  
  nc stellt die funktionen zum bildschirmaufbau mit ncurses
  
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <curses.h>
#include "tefaxdef.h"
#include "debug.h"
#include "hul.h"
#include "hcs.h"
#include "hsp.h"
#include "hlf.h"


/* ausgabe einer -fehler- meldung von max zwei zeilen im listenfenster 
** die strings werden in e und d uebergeben und enthalten normalerweise
** den error und das todo, falls die zeiger NULL sind werden die 
** voreinstellungen error_occured und do_nothing verwendet
*/
void printinerr(char *e, char *d)
{
  static WINDOW *w = NULL;
  static char fline[160];

  CF(printinerr);
  if (!w) w = newwin(daterfenster);

  sprintf(fline, "\n\t%s\n\t%s", 
	  (e) ? cutonend(e, 70) : S_(er_eroc),
	  (d) ? cutonbeg(d, 70) : S_(er_dono));
  
  wclear(w);
  waddstr(w, fline);
#ifndef NONC 
  wrefresh(w);
#else
  printf(fline, "\n\t%s\n\t%s\n", 
	 (e) ? cutonend(e, 70) : S_(er_eroc),
	 (d) ? cutonbeg(d, 70) : S_(er_dono));
#endif  
  CE;
}


/* printinsel formatiert die daten aus den nodes von root mit der funktion
** pf und bewegt den auswahlbalken entsprechend dem wert in mv 
*/
void printinsel(struct unilist *root, int mv, char *(*pf)(struct unilist *))
{
  static WINDOW *liste = NULL;
  static int lastlen;
  
  int basex = 0, i = 0;
  struct unilist *node = NULL;

  CF(printinsel);
  if (!root) { 
    printinerr(S_(er_eroc), S_(ind_no_ent));
    CE; return; }		/* freepad bei NULL,0, NULL ? */
  if(!root->next) { 
    printinerr(S_(ind_tare), S_(ind_no_ent));
    CE;  return;  }	/* msg bei leerer liste mit root->desc */

  if (root->cnt != lastlen || !root->nextsel) {
    mv = newlist;
    lastlen = root->cnt;
  }

  /* vorbereitung der liste fuer findsel damit die funktion den gewuenschten
  ** eintrag findet. wenn root->nextsel belegt ist, gilt dieser eintrag als
  ** auswahl. andernfalls versucht findsel den eintrag ueber stringoperationen
  ** zu finden bzw. selektiert den ersten eintrag in der liste (root->next)
  */
  switch (mv) {
  case move_up:
    if (root->nextsel->nextsel) root->nextsel = root->nextsel->nextsel;
    findsel(root);
    break;
  case move_down:
    if (root->nextsel->next) root->nextsel = root->nextsel->next;
    findsel(root);
    break;
  case newlist:
    /* wenn die liste neu aufgebaut werden muss, hat sich evtl. die laenge
    ** geaendert */
    if (liste) delwin(liste);
    findsel(root);
    liste = newpad((root->cnt > listlines) ? root->cnt +1 : listlines +1 , 80);
    break;
  }
  if (liste) {
    /* eintraege in das pad (liste) schreiben. der root->nextsel zeigt auf den 
    ** selektierten eintrag und die bildschirmattribute werden an der ent-
    ** sprechenden stelle geandert */
    if (((node = root)) && (root->day > 0) && (root->nextsel)) {
      wclear(liste);
      while (liste && ((node = node->next))) {
	if (node == root->nextsel) wattron(liste, A_REVERSE);
	waddstr(liste, pf(node));
	if (node == root->nextsel) wattroff(liste, A_REVERSE);
#ifdef NONC  
	if (node == root->nextsel)
	  printf("%s", pf(node));
#endif
      }
    }

    /* das ausgabefenster ueber der liste wird so verschoben, dass der der
    ** auswahlbalken soweit als moeglich in die mitte geschoben wird */
    i = listlines;
    basex = root->day - (i / 2);
    if ((basex + i) > root->cnt) basex = root->cnt - i;
    if (basex < 0) basex = 0;
#ifndef NONC  
    prefresh(liste, listfenster);
#endif
    }
  CE;
}  


/* printinpad formatiert die daten aus den nodes von root mit der funktion
** pf und bewegt das fenster ueber dem pad entsprechend dem wert in mv 
*/
void printinpad(struct unilist *root, int mv, char *(*pf)(struct unilist *))
{
  static WINDOW *liste = NULL;
  static int basex = 0;
#ifdef NONC  
  int fpl;			/* first printed line - erste angezeigte zeile */
#endif
  
  CF(printinpad);
  if (!root) { 
    printinerr(S_(er_eroc), S_(ind_no_ent));
    CE; return; }		/* freepad bei NULL,0, NULL ? */
  if(!root->nextsel) { 
    printinerr(S_(ind_tare), S_(ind_no_ent));
    CE;  return;  }	/* msg bei leerer liste mit root->desc */
  
  /* ausgabefenster ueber dem pad positionieren */
  switch (mv) {
  case move_up:
    if (basex > 0) basex--;

    /* die ausgabe ohne ncurses efolgt bei move_up und move_down
    ** ab zeile basex bis zum ende der liste */
#ifdef NONC  
    for (fpl = 0; fpl++ < basex && root->nextsel; root = root->nextsel);
    while ((root = root->nextsel))
      printf("%s", pf(root));
#endif
    break;
  case move_down:
    if (basex + listlines < root->res) basex++;
#ifdef NONC  
    for (fpl = 0; fpl++ < basex && root->nextsel; root = root->nextsel);
    while ((root = root->nextsel))
      printf("%s", pf(root));
#endif
    break;
  case newlist:
    /* wenn die liste neu aufgebaut werden muss, hat sich evtl. die laenge
    ** geaendert */
    if (liste) delwin(liste);
    basex = 0;
    liste = newpad((root->res > listlines) ? root->res +1 : listlines +1, 80);

    /* im gegensatz zu printinsel muss die liste nur neu geschrieben werden, wenn
    ** sich der inhalt geandert hat, sonst genuegt ein verschieben des ausgabe-
    ** fensters */
    while (liste && ((root = root->nextsel))) {
      waddstr(liste, pf(root));
#ifdef NONC  
      printf("%s", pf(root));
#endif
    }
    break;
  }
#ifndef NONC  
  if (liste)
    prefresh(liste, listfenster);
#endif
  CE;
  return;
}


/* printaline teilt den bildschirm in fenster zu je einer zeile
** diese funktion dient speziell der ausgabe der einzelnen menues
** sowie der statuszeile 
*/
WINDOW *printaline(int l, char *s)
{
  static WINDOW *wline[24];
  static int arinit;

  CF(printaline);
  
  if (l < 0 || l > 23) { CE; return NULL; }

  /* das array mit den enzelnen windows initialisieren */
  if (!arinit)
    while (arinit < 24) wline[arinit++] = NULL;

  /* ueber den fenstern in denen eingaben moeglich sind muessen mit keypad
  ** die funktionstasten erlaubt werden. durch die freigabe von scrollok
  ** wird das loeschen der zeilen mit \n ermoeglicht */
  if (!wline[l]) { 
    wline[l] = newwin(1,80,l,0);
    scrollok(wline[l], TRUE);
    if (l == line_co || l == line_af || l == line_an || l == line_tn) 
      keypad(wline[l], TRUE);
  }
  
  /* die durch l benannte zeile loeschen und mit dem string in s belegen */
  if (s && wline[l]) {
    waddstr(wline[l], "\n");
    wclrtoeol(wline[l]);
    waddstr(wline[l], s);
  }
#ifndef NONC 
  if (wline[l]) wrefresh(wline[l]);
#else
  if (s) printf("\n%s\n", s);
#endif
  CE;
  return wline[l];
}

/* print_a_centered_line zentriert den string s vor der uebergabe an
** printaline mit fillforcenter */
WINDOW *pacline(int l, char *s)
{
  CF(pacline); CE;
  if (s)
    if (strlen(s) < 81) return printaline(l, fillforcenter(s));
    else return printaline(l, fillforcenter(S_(er_eroc)));
  else return printaline(l, fillforcenter(S_(er_eroc)));
}

/* print_formattet_in_line:  ausgabefunktion fuer die shellaufrufe
** die daten werden entweder ->full entnommen oder ->name wird mit 
** newpath verkettet und in anfuerungszeichen eingeschlossen durch 
** leerzeichen getrennt in einem string zurueckgegeben
** der speicher wird mit malloc nach bedarf angefordert bzw. erweitert 
*/
char *print_fil(char *newpath, struct unilist *root)
{
  int i = BUFSIZ;		        /* alloc length */
  char *ncl = (char *) malloc(i);	/* new command line */
  char *nal;			        /* new allocated line */

  CF(print_fil);

  if (!root || !root->next || !ncl) { CE; return NULL; }
  ncl[0] = '\0';
  while((root = root->next)) {
    if (strlen(root->full) + strlen(ncl) + BUFSIZ > i) {
      nal = (char *) malloc(i += BUFSIZ);
      if (!nal) { free(ncl); CE; return ncl = NULL; }
      strcpy(nal, ncl);
      free(ncl);
      ncl = nal;
    }
    if (ncl[0] == '\0') strcat(ncl, "\"");
    else strcat(ncl, " \"");
    if (newpath) {
      strcat(ncl, newpath);
      strcat(ncl, root->name);
    }
    else
      strcat(ncl, root->full);
    strcat(ncl, "\"");
  }
  CE;
  return ncl;
}

/* print_formattet_link_line:  ausgabefunktion fuer die shellaufrufe
** die daten werden entweder ->full und ->name entnommen und durch
** anfuehrungszeichen (wegen der leerzeichen) geschuetzt zu link-
** befehlen erweitert, die durch semicolon getrennt sind.
** das ergebnis ist je ein symbolischer link bezeichnet aus og_path
** und ->name auf ->full. uebergabe der link-befehls-liste 
** in mm.c: mm_send() an eine (r)shell
** der speicher wird mit malloc nach bedarf angefordert bzw. erweitert 
*/
char *print_fll(char *newpath, struct unilist *root)
{
  int i = BUFSIZ;		        /* alloc length */
  char *ncl = (char *) malloc(i);	/* new command line */
  char *nal;			        /* new allocated line */

  CF(print_fil);

  if (!root || !root->next || !ncl) { CE; return NULL; }
  ncl[0] = '\0';
  while((root = root->next)) {
    if (strlen(root->full) + strlen(ncl) + BUFSIZ > i) {
      nal = (char *) malloc(i += BUFSIZ);
      if (!nal) { free(ncl); CE; return ncl = NULL; }
      strcpy(nal, ncl);
      free(ncl);
      ncl = nal;
    }
    if (ncl[0] == '\0') strcat(ncl, "ln -s \"");
    else strcat(ncl, "; ln -s \"");
    strcat(ncl, root->full);
    strcat(ncl, "\" \"");
    strcat(ncl, newpath);
    strcat(ncl, root->name);
    strcat(ncl, "\"");
  }
  CE;
  return ncl;
}

/* userinput erweitert das eingegebene zeichen zu dem vollen befehl und
** zeigt ihn an. das eingegeben zeichen aus wgetch wird an die 
** aufrufende funktion zurueckgegeben */
int userin()
{
  int userinput;
  WINDOW *wcom = NULL;
 
  CF(userin);
  halfdelay(255);
  wcom = printaline(line_co, S_(mnu_cmd));
#ifndef NONC
  userinput = wgetch(wcom);
#else
  userinput = getchar();
#endif
  
  switch (userinput) {
  case MK_SEND:
    waddstr(wcom, S_(mna_send));
    break;
  case MK_SEND_ED:
    waddstr(wcom, S_(mna_send));
    break;
  case MK_FORWARD_SF:
    waddstr(wcom, S_(mna_f_fax));
    break;
  case MK_FORWARD:
    waddstr(wcom, S_(mna_f_file));
    break;
  case MK_CALLBACK:
    waddstr(wcom, S_(mna_c_back));
    break;
  case MK_CALLBACK_ED:
    waddstr(wcom, S_(mna_c_back));
    break;
  case MK_DELETE:
    waddstr(wcom, S_(mna_d_fax));
    break;
  case MK_NAME_DEL:
    waddstr(wcom, S_(mna_d_name));
    break;
  case MK_APPEND:
    waddstr(wcom, S_(mna_a_file));
    break;
  case MK_PRINT_SF:
    waddstr(wcom, S_(mna_p_fax));
    break;
  case MK_PRINT:
    waddstr(wcom, S_(mna_p_file));
    break;
  case MK_VIEW_SF:
    waddstr(wcom, S_(mna_v_fax));
    break;
  case MK_VIEW:
    waddstr(wcom, S_(mna_v_file));
    break;
  case MK_G3_SF:
    waddstr(wcom, S_(mna_g_fax));
    break;
  case MK_G3:
    waddstr(wcom, S_(mna_g_file));
    break;
  case MK_READ:
    waddstr(wcom, S_(mna_r_fax));
    break;
  case MK_QUIT:
    waddstr(wcom, S_(mna_quit));
    break;
  }
#ifndef NONC
  wrefresh(wcom);
#endif
  nocbreak();
  CE;
  return userinput;
}





