/**************************************************************************

    tefax  -- front end to handle FAXes with efax or mgetty&sendfax
              and certain programms for view and print

    Copyright (C) 1997  Kai Uwe Tempel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    please send any bug reports, suggestions, greetings etc. to
	temu@server3.medizin.uni-leipzig.de

    or contact me in
	Kai Uwe Tempel, Gaertnerstr. 171, 04209 Leipzig, Germany

    call or fax: +49 341 4126014

***************************************************************************/
/****************************************************************************

tefax main menu code file

***************************************************************************/

/*LINTLIBRARY */

#include <stdio.h>
#include <curses.h>
#include "tefaxdef.h"
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "debug.h"
#include "hdw.h"
#include "hul.h"
#include "hcs.h"
#include "hnc.h"
#include "hrc.h"
#include "hsp.h"
#include "hsn.h"
#include "hsf.h"
#include "hmm.h"
#include "hlf.h"

/******************************************************************************
******************************************************************************/

/* switchfaxbox() is the only funktion that recides in the main file - cause only
** main() use it. it switches the directory is readed as faxbox. the directrys
** are defined in cs.c with names pth_ic and pth_del. it also assumes that pth_del
** is on the local machine and pth_ic may be on a remote machine, so server will
** set to co_server if pth_ic is active
*/

char *switchfaxbox(char **server, char **box)
{
  int  ls;                  /** integer fuer die laengen der zeichenketten server
			    *** pth_ic und mnu_top zur berechnung der zeilenlaenge
			    **/
  int  li;
  int  lt = strlen(S_(mnu_top));
  char *pic = NULL;         /* pointer -> boxname zur anzeige ggf. gekuerzt */
  char *psv = NULL;         /* pointer -> servername zur anzeige ggf. gekuerzt */
  static char topline[81];  /* die neue topline */

  CF(switchfaxbox);

/* der eintrag fuer die faxbox wechselt zwischen den beiden eintraegen in pth_ic
** in pth_del, die in cs.c definiert sind. der eintrag fuer den server wird bei
** aktiver recived-box auf NULL gesetzt, da diese immer lokal ist. wenn das 
** eingangsverzeichnis gelesen wird nimmt server den wert von co_server an
*/
  if (*box == S_(pth_del)) {
    *server = S_(co_server);
    *box = S_(pth_ic);
  }
  else {
    *server = NULL;
    *box = S_(pth_del);
  }

/* und nun noch eine schoene kopfzeile mit der aktuellen faxbox incl. server */
  li = strlen(*box);
  if (*server) {
    ls = strlen(*server);
    if ((li + ls) > (80 - lt - 10)) {
      if (li < ls) {
	if (li > 80 - lt - 10 - 30) {
	  pic = cutonbeg(*box, 80 - lt - 10 - 30);
	  li = strlen(pic);
	}
	psv = cutonend(*server, 80 - lt - li - 10);
      }
      else {
	if (ls > 80 - lt - 30 - 10) {
	  psv = cutonend(*server, 80 - lt - 30 - 10);
	  ls = strlen(psv);
	}
	pic = cutonbeg(*box, 80 - lt - 10 - ls);
      }
    }
    else { 
      pic = *box;
      psv = *server;
    }
    sprintf(topline, "%s %s on %s", S_(mnu_top), pic, psv);
  }
  else
    sprintf(topline, "%s %s", S_(mnu_top), cutonbeg(*box, 76 - lt));

  CE;
  return topline;
}

/*
** main verwaltet im prinzip nur das hauptmenue
*/
int main(int argc, char *argv[])
{
  int ck, foundarg = 0;	  /* check command line parameter */
  char *optconfig = NULL; /* extra variable for opt. config file */
  char *my_name = *argv;  /* name for recursive call under X11 */
  char xcl[BUFSIZ];       /* commandline for recursive call under X11 */

  struct unilist *forsend = getuninode();
  struct unilist *faxlist = getuninode();
  int (*sortby)(const void *, const void *) = compname; /* preset, should bo done
							 in configfile */
  char *server  = NULL;
  char *faxbox  = NULL;
  char *topline = NULL;
  int newscan   = 1;
  int userinput = 0;
  int setres    = 0;

/* der anfang des rekursiven aufrufs unter X11 */
  sprintf(xcl, "%s %s -x", S_(co_x_term), my_name);

/* alle kommandozeilenargumente lesen (nach K&R), das bitfeld in tefax_opt() 
** belegen und die argumente ausserdem an xcl anhaengen
*/
  while (--argc > 0 && **++argv == '-' && !(foundarg = 0)) {
    strcat(xcl, " ");
    strcat(xcl, *argv);
    while ((ck = *++argv[0]) && foundarg == 0)
      switch (ck) {
      case 'a': O_SET(wo_autores); break;           /* set autores */
      case 'A': O_DEL(wo_autores); break;	           
      /* -c CONFIGFILE */
      case 'C':	O_SET(ARG_S); break;                /* print all settings */
      case 'd':	O_SET(ARG_d); break;                /* home = cwd */
      /* -f FAXDIR FILE */
      case 'F': O_DEL(avl_fax); break;              /* no fax script */
      case 'G': O_DEL(avl_ghost); break;            /* no ghostview */
      case 'h':	O_SET(ARG_h); break;                /* help */
      case 'i': O_SET(wo_mem_ic); break;            /* set memic */
      case 'I': O_DEL(wo_mem_ic); break;	           
      case 'l':	O_SET(ARG_l); break;                /* run local */
      case 'm':	O_SET(ARG_i); break;                /* mail processing */
      case 'M': O_DEL(avl_sendfax); break;	           
      case 'n': O_SET(wo_preset_no); break;         /* set preset number */
      case 'N': O_DEL(wo_preset_no); break;	           
      case 'o': O_SET(wo_mem_og); break;            /* set mem og */
      case 'O': O_DEL(wo_mem_og); break;	           
      case 'p': O_SET(wo_proxy); break;	            /* set proxy */
      case 'P': O_DEL(wo_proxy); break;	           
      case 'q': O_DEL(wo_needconfirm); break;       /* set needconfirm */
      case 'Q': O_SET(wo_needconfirm); break;	           
      case 'r': O_SET(wo_rm_pxx); break;	    /* remove ps- and pm-files */
      case 'R': O_DEL(wo_rm_pxx); break;	           
      case 's': O_DEL(wo_p_check); break;           /* no programm check */
      case 'S': O_SET(wo_p_check); break;	           
      case 'V': O_DEL(avl_xv); break;               /* no xv */
      case 'v': printf("%s", "tefax, version 1.01, march 1997\n"); exit(0); break;
      case 'W': gnutextout(); exit(0); break;
      case 'w':	O_SET(ARG_n); break;                /* execute nothing */
      case 'x': O_SET(ARG_x); break;                /* falls -x: kein neues fenster */
      case 'c':                                     /* alternative configuration file */
	if (--argc > 0 && **++argv != '-') {
	  strcat(xcl, " "); 
	  strcat(xcl, (optconfig = *argv));
	  foundarg = 1; 
	}
	break;
      case 'f':                                     /* alternative faxdir-file */
	if (--argc > 0 && **++argv != '-') {
	  strcat(xcl, " "); 
	  strcat(xcl, *argv);
	  S_S(fl_faxdirfile, *argv); 
	  foundarg = 1; 
	}
	break;
      default:                                      /*  ungueltige parameter */
 	printhelptext(my_name); 
	exit(2); 
	break;
      }
  }

  if (argc != 0 || O_GET(ARG_h)) {                  /* falls ungueltige parameter */
    printhelptext(my_name);
    exit(2);
  }

  if (O_GET(ARG_i)) {                               /* mail processing */
    setup_var(optconfig);
    seeincoming();
    exit(0);
  }

  if (O_GET(ARG_S)) {                               /* print all settings */
    setup_var(optconfig);
    printallsettings();
    exit(0);
  }

/* wenn WINDOWID mindestens zweistellig ist, tefax in enem x-term starten*/
  if (getenv("WINDOWID") && strlen(getenv("WINDOWID")) > 1 && !O_GET(ARG_x)) {
    shellxx(xcl);
    exit(0);
  }

/* configfile einlesen */
  setup_var(optconfig);

/* faxbox auf recived folder, damit nach dem ersten aufruf zur initialisierung
** das eingangsverzeichnis eingelesen wird
*/
  faxbox = S_(pth_del);   

/* ncurses initialisieren */
  initscr();
  noecho();

/* aufbau des hauptmenues */
  pacline(line_tl, (topline = switchfaxbox(&server, &faxbox)));
  pacline(line_av, S_(mnu_av));
  if (O_GET(ARG_x) && O_GET(avl_g3view)) pacline(line_c1, S_(mnu_1gx));
  else if (!O_GET(ARG_x) && O_GET(avl_g3vga)) pacline(line_c1, S_(mnu_1g));
  else pacline(line_c1, S_(mnu_1));
  
/* faxliste aus den dateien im eingangsverzeichnis aufbauen */
  pacline(line_st, S_(sts_ri));
  getdire(server, faxbox, faxlist);
  striptofaxlist(faxlist, server, S_(fl_faxdirfile), S_(fm_readfaxdir));
  selsort(faxlist, sortby);

/* am anfang ist immer der erste eintrag selektiert: faxlist->next, 
** dadurch kann das menue komplett aufgebaut werden, bevor die liste 
** geschrieben wird 
*/
  if (faxlist->next && (faxlist->next->res == 'N')) {
    O_DEF(ARG_L);
    pacline(line_c2, S_(mnu_2h));
  }
  else pacline(line_c2, S_(mnu_2l));

/* faxliste auf dem bildschirm ausgeben, falls leere liste meldung */
  if (faxlist->next) printinsel(faxlist, newlist, pn_fax);
  else printinerr(S_(ind_tare), S_(ind_no_fax));

/* das eigentliche menue bis ende durch quit */
  while (((userinput = userin())) != MK_QUIT) {
    switch (userinput) {
    case ERR:            /* nach 20 sec. wird das verzeichnis neu eingelesen */
      printaline(line_st, "\n");
      newscan = 1;
      break;
    case MK_HELP:
      printmainhelp();
      pacline(line_tl, topline);
      pacline(line_av, S_(mnu_av));
      newscan = 1;
      break;
    case MK_WARR:
      print_gnu();
      pacline(line_tl, topline);
      pacline(line_av, S_(mnu_av));
      newscan = 1;
      break;
    case MK_SHELL:
      clear();
      refresh();
      echo();
      shellxx("bash");
      refresh();
      noecho();
      pacline(line_tl, topline);
      pacline(line_av, S_(mnu_av));
      newscan = 1;
      break;
    case MK_UP:          /* elm-kompatibel move */
      if (faxlist->next)
	printinsel(faxlist, move_up, pn_fax);
      setres = 1;
      break;
    case MK_DOWN:        /* elm-kompatibel move */
      if (faxlist->next)
	printinsel(faxlist, move_down, pn_fax);
      setres = 1;
      break;
    case KEY_UP:         /* cursor-tasten */
      if (faxlist->next)
	printinsel(faxlist, move_up, pn_fax);
      setres = 1;
      break;
    case KEY_DOWN:       /* cursor-tasten */
      if (faxlist->next)
	printinsel(faxlist, move_down, pn_fax);
      setres = 1;
      break;
    case MK_BOX_IN:      /* faxbox wechseln */
      /* damit findsel in der faxliste nicht die eintraege aus dem anderen
      ** verzeichnis sucht (und natuerlich nicht findet) muss sichergestellt
      ** werden, dass die lidte nicht nur neu aufgebaut, sondern komplett
      ** als neue liste wie beim programmstart betrachtet wird */
      free(faxlist->name); faxlist->name = NULL;
      faxlist->nextsel = NULL;
      pacline(line_tl, (topline = switchfaxbox(&server, &faxbox)));
      newscan = 1;
      setres = 1;
      break;
    case MK_BOX_DEL:     /* faxbox wechseln */
      /* damit findsel in der faxliste nicht die eintraege aus dem anderen
      ** verzeichnis sucht (und natuerlich nicht findet) muss sichergestellt
      ** werden, dass die lidte nicht nur neu aufgebaut, sondern komplett
      ** als neue liste wie beim programmstart betrachtet wird */
      free(faxlist->name); faxlist->name = NULL;
      faxlist->nextsel = NULL;
      pacline(line_tl, (topline = switchfaxbox(&server, &faxbox)));
      newscan = 1;
      setres = 1;
      break;
    case MK_SORT_N:      /* faxliste nach dateinamen sortieren */
      if (faxlist->next) {
	sortby = compname;
	selsort(faxlist, sortby);
	printinsel(faxlist, newlist, pn_fax);
      }
      break;
    case MK_SORT_D:      /* faxliste nach absender sortieren */
      if (faxlist->next) {
	sortby = compdesc;
	selsort(faxlist, sortby);
	printinsel(faxlist, newlist, pn_fax);
      }
      break;
    case MK_SORT_T:      /* faxliste nach eingangsdatum sortieren */
      if (faxlist->next) {
	sortby = compdate;
	selsort(faxlist, sortby);
	printinsel(faxlist, newlist, pn_fax);
      }
      break;
    case MK_SORT_NR:     /* faxliste reverse nach dateinamen sortieren */
      if (faxlist->next) {
	sortby = comp_name;
	selsort(faxlist, sortby);
	printinsel(faxlist, newlist, pn_fax);
      }
      break;
    case MK_SORT_DR:     /* faxliste reverse nach absender sortieren */
      if (faxlist->next) {
	sortby = comp_desc;
	selsort(faxlist, sortby);
	printinsel(faxlist, newlist, pn_fax);
      }
      break;
    case MK_SORT_TR:     /* faxliste reverse nach eingangsdatum sortieren */
      if (faxlist->next) {
	sortby = comp_date;
	selsort(faxlist, sortby);
	printinsel(faxlist, newlist, pn_fax);
      }
      break;
    case MK_HIGH:        /* aufloesung auf fine setzen */
      pacline(line_st, S_(sts_high));
      O_UND(ARG_L);
      break;
    case MK_LOW:         /* aufloesung auf normal setzen */
      pacline(line_st, S_(sts_low));
      O_DEF(ARG_L);
      break;
    case MK_NAME_DEL:    /* empfaenger aus faxdir-file loeschen --
			  die uebergabe von hl_del_no veranlasst das loeschen
			  des namens aus dem faxdir-file */
      selectname(S_(fl_faxdirfile), forsend, S_(hl_del_no));
      newscan = 1;
      break;
    case MK_DELETE:      /* fax nach recived folder verschieben bzw. loeschen
			  die dateien des fax werden in der forsend-liste ge-
			  sammelt, incl. tefax-hilfsdatei und abhaengig von der
			  variablen faxbox geloescht oder verschoben.
			  anschiessend wird der eintrag aus der faxliste entfernt
			  und die liste neu ausgegeben (ggf. leere liste) */
      if (faxlist->nextsel) {
	preset_fsl(server, faxbox, forsend, faxlist->nextsel, select_fax);
	if (faxbox == S_(pth_ic)) mm_delete(server, forsend);
	else mm_kill(faxbox, forsend);
	remuninode(faxlist, faxlist->nextsel);
	if (faxlist->next)
	  printinsel(faxlist, newlist, pn_fax);
	else printinerr(S_(ind_tare), S_(ind_no_fax));
	setres = 1;
      }
      break;
    case MK_SEND:        /* empfaenger und dateien auswaehlen und senden */
      newscan = 1;
      if (!selectname(S_(fl_faxdirfile), forsend, S_(hl_send_to))) {
	pacline(line_st, S_(sts_nos));	break;      }
      if (!prepare_fsl(NULL, S_(pth_og), forsend, S_(hl_send_f))) {
	pacline(line_st, S_(sts_nos));	break;      }
      mm_send(NULL, forsend);
      pacline(line_st, S_(sts_sent));
      break;
    case MK_SEND_ED:     /* wie SEND, zusaetzlich editor aufrufen */
      newscan = 1;
      if (!selectname(S_(fl_faxdirfile), forsend, S_(hl_send_to))) {
	pacline(line_st, S_(sts_nos));	break;      }
      if (!invocingeditor(forsend)) {
	pacline(line_st, S_(sts_nos));	break;      }
      if (!prepare_fsl(NULL, S_(pth_og), forsend, S_(hl_send_f))) {
	pacline(line_st, S_(sts_nos));	break;      }
      mm_send(NULL, forsend);
      pacline(line_st, S_(sts_sent));
      break;
    case MK_CALLBACK:    /* absender des selektierten fax und seine faxnummer
			    in die wurzel der forsendliste eintragen, als vor-
			    einstellung fuer selectname. weiter wie SEND */
      newscan = 1;
      free(forsend->desc); forsend->desc = NULL;
      if (faxlist->full) forsend->desc = strdup(faxlist->full);
      free(forsend->name); forsend->name = NULL;
      if (faxlist->desc) forsend->name = strdup(faxlist->desc);
      if (!selectname(S_(fl_faxdirfile), forsend, S_(hl_call_back))) {
	pacline(line_st, S_(sts_nos));	break;      }
      if (!prepare_fsl(NULL, S_(pth_og), forsend, S_(hl_send_f))) {
	pacline(line_st, S_(sts_nos));	break;      }
      mm_send(NULL, forsend);
      pacline(line_st, S_(sts_sent));
      break;
    case MK_CALLBACK_ED: /* wie CALLBACK, zusaetzlich aufruf editor */
      newscan = 1;
      free(forsend->desc); forsend->desc = NULL;
      if (faxlist->full) forsend->desc = strdup(faxlist->full);
      free(forsend->name); forsend->name = NULL;
      if (faxlist->desc) forsend->name = strdup(faxlist->desc);
      if (!selectname(S_(fl_faxdirfile), forsend, S_(hl_call_back))) {
	pacline(line_st, S_(sts_nos));	break;      }
      if (!invocingeditor(forsend)) {
	pacline(line_st, S_(sts_nos));	break;      }
      if (!prepare_fsl(NULL, S_(pth_og), forsend, S_(hl_send_f))) {
	pacline(line_st, S_(sts_nos));	break;      }
      mm_send(NULL, forsend);
      pacline(line_st, S_(sts_sent));
      break;
    case MK_FORWARD_SF:  /* die dateien des selektierten fax in die forsend-
			    liste eintragen, empfaenger auswaehlen lassen.
			    die forsend-liste zum editieren an prepare_fsl
			    uebergeben und schliesslich verschicken */
      if (faxlist->nextsel) {
       	newscan = 1;
	preset_fsl(server, faxbox, forsend, faxlist->nextsel, select_tor);
	if (!selectname(S_(fl_faxdirfile), forsend, S_(hl_forw_t))) {
	  pacline(line_st, S_(sts_nos));  break;    }
	if (!prepare_fsl(server, faxbox, forsend, S_(hl_forw_f))) {
	  pacline(line_st, S_(sts_nos));  break;    }
	mm_send(server, forsend);
	pacline(line_st, S_(sts_sent));
	break;
      }
    case MK_FORWARD:     /* wie FORWARD_SF, jedoch ohne vorbelegung der
			    forsend-liste */
      newscan = 1;
      if (!selectname(S_(fl_faxdirfile), forsend, S_(hl_forw_t))) {
	pacline(line_st, S_(sts_nos));	break;      }
      if (!prepare_fsl(server, faxbox, forsend, S_(hl_forw_f))) {
	pacline(line_st, S_(sts_nos));	break;      }
      mm_send(server, forsend);
      pacline(line_st, S_(sts_sent));
      break;

/* die folgenden memuepunkte zur anzeige von faxdateien unterscheiden
** unterscheiden sich nur durch die aufgerufenen programme und die uebergebenen
** parameter. das anzeigeprogramm wird nach der verfuegbarkeit ausgewaehlt sowie
** abhaengig davon ob X11 laeuft. die macros in mm.c entscheiden anhand des bits
** ARG_L in tefax_opt ueber die parameter. die dateiliste enthaelt entweder alle
** dateien des aktuellen faxes, oder sie wird manuell durch selectfile zusammen-
** gestellt
 */
    case MK_G3_SF:
      if (faxlist->nextsel) {
	newscan = 1;	
	if (preset_fsl(server, faxbox, forsend, faxlist->nextsel, select_tor)) {
	  if (O_GET(ARG_x) && (O_GET(avl_g3view))) mm_g3x(server, forsend);
	  if (!O_GET(ARG_x) && (O_GET(avl_g3vga))) mm_g3c(server, forsend);
	}
      }
      break;
    case MK_G3:
      if (faxlist->nextsel) {
	char *helpline = (O_GET(ARG_L)) ? S_(hl_show_h) : S_(hl_show_l);
	newscan = 1;
	if (selectfile(server, faxbox, forsend, helpline)) {
	  if (O_GET(ARG_x) && (O_GET(avl_g3view))) mm_g3x(server, forsend);
	  if (!O_GET(ARG_x) && (O_GET(avl_g3vga))) mm_g3c(server, forsend);
	}
      }
      break;
    case MK_VIEW_SF:
      free(forsend->name); forsend->name = NULL;
      if (faxlist->name) forsend->name = strdup(faxlist->name);
      if (faxlist->nextsel && O_GET(avl_efix)) {
	newscan = 1;
	if (preset_fsl(server, faxbox, forsend, faxlist->nextsel, select_tor)) {
	  if (O_GET(ARG_x)) {
	    if (O_GET(avl_xv)) {
	      char *convert = (O_GET(ARG_L)) ? S_(co_xl_view) : S_(co_xh_view);
	      mm_pgm(server, forsend, convert, S_(avl_xv));
	    }
	    else if (O_GET(avl_ghost)) {
	      char *convert = (O_GET(ARG_L)) ? S_(co_cl_print) : S_(co_ch_print);
	      mm_pps(server, forsend, convert, S_(avl_ghost));
	    }
	    else if (O_GET(avl_gs)) {
	      char *convert = (O_GET(ARG_L)) ? S_(co_cl_print) : S_(co_ch_print);
	      mm_pps(server, forsend, convert, S_(co_gs_x));
	    }
	  }
	  else {
	    char *convert = (O_GET(ARG_L)) ? S_(co_cl_view) : S_(co_ch_view);
	    if (O_GET(avl_gs)) mm_pps(server, forsend, convert, S_(co_gs_view));
	  }
	}
      }
      break;
    case MK_VIEW:
      if (O_GET(avl_efix)) {
	char *helpline = (O_GET(ARG_L)) ? S_(hl_show_h) : S_(hl_show_l);
	newscan = 1;
	if (selectfile(server, faxbox, forsend, helpline)) {
	  if (O_GET(ARG_x)) {
	    if (O_GET(avl_xv)) {
	      char *convert = (O_GET(ARG_L)) ? S_(co_xl_view) : S_(co_xh_view);
	      mm_pgm(server, forsend, convert, S_(avl_xv));
	    }
	    else if (O_GET(avl_ghost)) {
	      char *convert = (O_GET(ARG_L)) ? S_(co_cl_print) : S_(co_ch_print);
	      mm_pps(server, forsend, convert, S_(avl_ghost));
	    }
	    else if (O_GET(avl_gs)) {
	      char *convert = (O_GET(ARG_L)) ? S_(co_cl_print) : S_(co_ch_print);
	      mm_pps(server, forsend, convert, S_(co_gs_x));
	    }
	  }
	  else {
	    char *convert = (O_GET(ARG_L)) ? S_(co_cl_view) : S_(co_ch_view);
	    if (O_GET(avl_gs)) mm_pps(server, forsend, convert, S_(co_gs_view));
	  }
	}
      }
      break;
    case MK_READ_SF:
      free(forsend->name); forsend->name = NULL;
      if (faxlist->name) forsend->name = strdup(faxlist->name);
      if (faxlist->nextsel && O_GET(avl_efix)) {
	newscan = 1;	
	if (preset_fsl(server, faxbox, forsend, faxlist->nextsel, select_tor)) {
	  if (O_GET(ARG_x)) {
	    if (O_GET(avl_xv)) {
	      char *convert = (O_GET(ARG_L)) ? S_(co_xl_read) : S_(co_xh_read);
	      mm_pgm(server, forsend, convert, S_(avl_xv));
	    }
	    else if (O_GET(avl_ghost)) {
	      char *convert = (O_GET(ARG_L)) ? S_(co_cl_print) : S_(co_ch_print);
	      mm_pps(server, forsend, convert, S_(avl_ghost));
	    }
	    else if (O_GET(avl_gs)) {
	      char *convert = (O_GET(ARG_L)) ? S_(co_cl_print) : S_(co_ch_print);
	      mm_pps(server, forsend, convert, S_(co_gs_x));
	    }
	  }
	  else {
	    char *convert = (O_GET(ARG_L)) ? S_(co_cl_read) : S_(co_ch_read);
	    if (O_GET(avl_gs)) mm_pps(server, forsend, convert, S_(co_gs_read));
	  }
	}
      }
      break;
    case MK_READ:
      if (O_GET(avl_efix)) {
	char *helpline = (O_GET(ARG_L)) ? S_(hl_read_h) : S_(hl_read_l);
	newscan = 1;
	if (selectfile(server, faxbox, forsend, helpline)) {
	  if (O_GET(ARG_x)) {
	    if (O_GET(avl_xv)) {
	      char *convert = (O_GET(ARG_L)) ? S_(co_xl_read) : S_(co_xh_read);
	      mm_pgm(server, forsend, convert, S_(avl_xv));
	    }
	    else if (O_GET(avl_ghost)) {
	      char *convert = (O_GET(ARG_L)) ? S_(co_cl_print) : S_(co_ch_print);
	      mm_pps(server, forsend, convert, S_(avl_ghost));
	    }
	    else if (O_GET(avl_gs)) {
	      char *convert = (O_GET(ARG_L)) ? S_(co_cl_print) : S_(co_ch_print);
	      mm_pps(server, forsend, convert, S_(co_gs_x));
	    }
	  }
	  else {
	    char *convert = (O_GET(ARG_L)) ? S_(co_cl_read) : S_(co_ch_read);
	    if (O_GET(avl_gs)) mm_pps(server, forsend, convert, S_(co_gs_read));
	  }
	}
      }
      break;
    case MK_PRINT_SF:
      free(forsend->name); forsend->name = NULL;
      if (faxlist->name) forsend->name = strdup(faxlist->name);
      if (faxlist->nextsel && O_GET(avl_efix)) {
	newscan = 1;	
	if (preset_fsl(server, faxbox, forsend, faxlist->nextsel, select_tor)) {
	  char *convert = (O_GET(ARG_L)) ? S_(co_cl_print) : S_(co_ch_print);
	  mm_pps(server, forsend, convert, S_(cl_lpr));
	}
      }
      break;
    case MK_PRINT:
      if (O_GET(avl_efix)) {
	char *helpline = (O_GET(ARG_L)) ? S_(hl_prnt_l) : S_(hl_prnt_h);
	newscan = 1;
	if (selectfile(server, faxbox, forsend, helpline)) {
	  char *convert = (O_GET(ARG_L)) ? S_(co_cl_print) : S_(co_ch_print);
	  mm_pps(server, forsend, convert, S_(cl_lpr));
	}
      }
      break;
    } /* end switch input*/

/* die forsend-liste muss nun geleert werden */
    cleanunilist(forsend);

/* nach dem neueinlesen der faxbox muss der bildschirm frisch aufgebaut werden */
    if (newscan) {
      if (O_GET(ARG_x) && O_GET(avl_g3view)) pacline(line_c1, S_(mnu_1gx));
      else if (!O_GET(ARG_x) && O_GET(avl_g3vga)) pacline(line_c1, S_(mnu_1g));
      else pacline(line_c1, S_(mnu_1));

      printaline(line_an, "\n");
      printaline(line_tn, "\n");
      printaline(line_af, "\n");

      getdire(server, faxbox, faxlist);
      striptofaxlist(faxlist, server, S_(fl_faxdirfile), S_(fm_readfaxdir));
      if (faxlist->next) {
	selsort(faxlist, sortby);
	printinsel(faxlist, newlist, pn_fax);
      }
      else printinerr(S_(ind_tare), S_(ind_no_fax));
      newscan = 0;
    }

/* bei eingeschaltetem autores wird das bit fuer die aufloesung entsprechend
** der angabe im aktuellen faxeintrag gesetzt. das manuelle aendern der auf-
** loesung verhindert ueber die variable setres ein zuruecksetzen der 
** aufloesung bis zur naechten bewegung in der faxliste */
    if (setres && O_GET(wo_autores))
      if (faxlist->nextsel && (faxlist->nextsel->res == 'N')) O_DEF(ARG_L);
      else O_UND(ARG_L);
    pacline(line_c2, (O_GET(ARG_L)) ? S_(mnu_2h) : S_(mnu_2l));
    
    setres = 0;
  } /* end while input */
  
/* aufraeumen: listen freigeben, bildschirm loeschen und die ncurses-strukturen
** wieder freigeben, sowie das echo wieder einschalten, weil die shell sonst die
** folgenden eingaben nicht auf dem bildschirm darstellt und zum teil nicht mal
** mit dem kommando reset die normale ausgabe wiederhergestellt werden kann :-(
*/
  freeunilist(forsend);
  freeunilist(faxlist);

  clear();
  refresh();
  echo();
  endwin();
  exit(0);
}




