/**************************************************************************

    tefax  -- front end to handle FAXes with efax or mgetty&sendfax
              and certain programms for view and print

    Copyright (C) 1997  Kai Uwe Tempel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    please send any bug reports, suggestions, greetings etc. to
	temu@server3.medizin.uni-leipzig.de

    or contact me in
	Kai Uwe Tempel, Gaertnerstr. 171, 04209 Leipzig, Germany

    call or fax: +49 341 4126014

***************************************************************************/
/***************************************************************************

read configuration

***************************************************************************/

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syscall.h>
#include <sys/stat.h>
#include <ctype.h>
#include "tefaxdef.h"
#include "hcs.h"
#include "his.h"
#include "hsp.h"
#include "debug.h"
#include "hlf.h"

#ifdef DEBUG
#define D_CONFIG
#endif

#ifdef D_CONFIG
#define DV(s) dv(s);
#else
#define DV(s) /**/
#endif

/* readconfig liest die datei fp mit der formatangabe in fm_readconfig 
** und vergleicht das erste wort jeder zeilen mit den eintraegen in der
** ersten spalte von tefax_string in string_tab() (cs.c). bei ueberein-
** stimmung in einer zeile des arrays vor LOCKED wird der string nach dem 
** gleichheitszeichen in die zweite spalte des arrays eingetragen
** einmal gesetzte eintraege werden durch string_tab() vor dem ueberschreiben
** geschuetzt, wenn sich die zeile nicht im bereich CHANGEABLE befindet 
*/
void readconfig(FILE *fp)
{
  int k, l, i = 0;
  char buf[BUFSIZ];
  char index[30];
  char match[BUFSIZ];
  
  CF(readconfig);
  
  while (fgets(buf, BUFSIZ, fp) != NULL)
    if (buf[0] != '#' && !isspace(buf[0])) {
      
      /* das einlesen mit sscanf ermoeglicht durch die formatangabe eine einfache
      ** veraenderung der syntax im config-file */
      l = sscanf(buf, S_(fm_readconfig), index, match);

      if (l == 1 || l == 2) {
	
	/* leerzeichen am stringende ggf. entfernen */
	for (k = strlen(index); isspace(index[--k]) && k > 0; index[k] = '\0');
	
	if (S_P(i) && strcmp(index, S_P(SETABLE)) == 0 && l == 2) {

	  char *bp = match;

	  while (*bp != '\0' && *bp != '\n') {
	    switch (*bp) {
	    case 'a': O_SET(wo_autores); break;           /* set autores */
	    case 'A': O_DEL(wo_autores); break;	           
	    case 'd': O_SET(ARG_d); break;                /* home = cwd */
	    case 'F': O_DEL(avl_fax); break;              /* no fax script */
	    case 'G': O_DEL(avl_ghost); break;            /* no ghostview */
	    case 'i': O_SET(wo_mem_ic); break;            /* set memic */
	    case 'I': O_DEL(wo_mem_ic); break;	           
	    case 'l': O_SET(ARG_l); break;                /* run local */
	    case 'M': O_DEL(avl_sendfax); break;	           
	    case 'n': O_SET(wo_preset_no); break;         /* set preset number */
	    case 'N': O_DEL(wo_preset_no); break;	           
	    case 'o': O_SET(wo_mem_og); break;            /* set mem og */
	    case 'O': O_DEL(wo_mem_og); break;	           
	    case 'p': O_SET(wo_proxy); break;	            /* set proxy */
	    case 'P': O_DEL(wo_proxy); break;	           
	    case 'q': O_DEL(wo_needconfirm); break;       /* set needconfirm */
	    case 'Q': O_SET(wo_needconfirm); break;	           
	    case 'r': O_SET(wo_rm_pxx); break;	    /* remove ps- and pm-files */
	    case 'R': O_DEL(wo_rm_pxx); break;	           
	    case 's': O_DEL(wo_p_check); break;           /* no programm check */
	    case 'S': O_SET(wo_p_check); break;	           
	    case 'V': O_DEL(avl_xv); break;               /* no xv */
	    case 'w': O_SET(ARG_n); break;                /* execute nothing */
	    case 'x': O_SET(ARG_x); break;                /* falls -x: kein neues fenster */
	    }
	    bp++;
	  }
	  continue;
	}		

	/* keywords werden nur bis zur zeile LOCKED im array gesucht */
	for (i = 0; i < LOCKED; i++) {
	  
	  /* strings vergleichen */
	  if (S_P(i) && strcmp(index, S_P(i)) == 0) {
	    
	    /* leerzeichen am stringende ggf. entfernen */
	    if (l == 2) {
	      for (k = strlen(match); isspace(match[--k]) && k > 0; match[k] = '\0');
	      S_S(i, match);
	    }
	    else {
	      match[0] = '\0';
	      S_S(i, match);
	    }
	    break;
	  }
	}
      }
    }
  CE;
}


void dv(char *p)
{
  int i = 0;
  char buf[BUFSIZ];
  S(next, "***********************************************************************");
  while (i < LASTCOM) {
    sprintf(buf, "%s || %-30.30s %-30.30s", p, S_P(i) ? S_P(i) : "NUL", S_(i) ? S_(i) : "NUL");
    S(d, buf);
    i++;
  }
}

/* der zentrale punkt fuer einen korrekten start des programms
** nach dem suchen (und einlesen) des configfile werden die wichtigsten
** variablen ueberprueft und das bitfeld k in tefax_opt() (cs.c) mit
** den working_options gesetzt
*/
void setup_var(char *optconfig)
{
  char *homedir = NULL;
  char cwdpath[BUFSIZ];
  char *search_path = NULL;
  char *shell = NULL;
  char buf[BUFSIZ] = "";
  char *bp = NULL;			/* bufpointer */
  FILE *fp = NULL;
  int i, k, afc = 0, sok = 0, pok = 0, vok = 0;
  struct stat stbuf;

  CF(setup_var);
  DV("start");
  /* auslesen der umgebung */
  homedir = getenv("HOME");
  search_path = strdup(getenv("PATH"));
  shell = getenv("SHELL");
  if (shell) S_S(cl_shell, strdup(shell));

  getcwd(cwdpath, BUFSIZ);
  if (O_GET(ARG_d)) homedir = cwdpath;

  if (!homedir || strlen(cwdpath) == 0) {
    printf("%s\n", S_(er_home));
    getchar();
    exit(1);
  }

  /* besser waere, die nicht benoetigten erweiterungen zu ueberspringen
  ** aber dann wuerden bei der option -C printallsettings die falschen
  ** angaben erscheinen */
  sprintf(buf, "%s/%s", homedir, S_(fl_confighome));
  S_S(fl_confighome, buf);
  
  sprintf(buf, "%s/%s", S_(pth_etc), S_(fl_configetc));
  S_S(fl_configetc, buf);
  
  sprintf(buf, "%s/%s", cwdpath, S_(fl_configcwd));
  S_S(fl_configcwd, buf);
      
  /* der reihe nach wird das configfile nach der komandozeilenoption, 
  ** in HOME, pth_etc und im aktuellen verzeichnis gesucht */

				/* read config file in: */
  if (optconfig) {		/* command line */
    if ((fp = fopen(optconfig, "r")))
    readconfig(fp);
  }

  if (!fp) {			/* home */
    if ((fp = fopen(S_(fl_confighome), "r")))
      readconfig(fp);
  }
  if (!fp) {			/* etc */
    if ((fp = fopen(S_(fl_configetc), "r")))
      readconfig(fp);
    else {			/* cwd */
      if ((fp = fopen(S_(fl_configcwd), "r")))
	readconfig(fp);
    }
  }
  if (fp) {fclose(fp); fp = NULL;}
  else {
    printf("%s\n", S_(er_neit));
    printf("%s\n", S_(er_cfg1));
    printf("%s\n", S_(er_cfg2));
    printf("%s\n", S_(er_cfg3));
    printf("%s\n", S_(er_abort));
    if (getchar() == 'q') exit(1);
  }
  DV("cfile");

  /* complete path and file names */
  for (i = FIRSTPATH; i < LASTBIN; i++) {
    int repl = 0;

    /* erkennung ob die angaben relativ oder absolut sind durch das erste
    ** zeichen */
    if ((bp = S_(i)) && strlen(bp) > 0) {
      if (*bp == '~') repl += 1; 
      if (*bp == '.' && bp[1] == '/') repl += 2;

      /* an pfadnamen wird ein slash angehaengt, damit eine einfache 
      ** verbindung mit den dateinamen moeglich ist */
      if (i < LASTPATH && lastchar(bp) != '/') repl += 4;
      switch (repl) {
      case 1:
	sprintf(buf, "%s%s", homedir, bp + 1);
	S_C(i, buf);
	break;
      case 2:
	sprintf(buf, "%s%s", cwdpath, bp + 1);
	S_C(i, buf);
	break;
      case 4:
	sprintf(buf, "%s/", bp);
	S_C(i, buf);
	break;
      case 5:
	sprintf(buf, "%s%s/", homedir, bp + 1);
	S_C(i, buf);
	break;
      case 6:
	sprintf(buf, "%s%s/", cwdpath, bp + 1);
	S_C(i, buf);
	break;
      }

      /* zuetzt die erweiterung von namen im cwd */
      if (i < LASTFILE && *S_(i) != '/') {
	sprintf(buf, "%s/%s", cwdpath, S_(i));
	S_C(i, buf);
      }
    }      
  }
  DV("cmplFN");
  /* comlete pathnames for programs*/
  for (i = FIRSTPROG; i < LASTPROG; i++) {
    int repl = 0;

    if ((bp = S_(i)) && strlen(bp) > 0) {

      /* die erweiterung von prorammnamen erfolgt nur bei relativen angaben
      ** da die programme sich ueblicherweise im suchpfad befinden */
      if (*bp == '~') repl += 1; 
      if (*bp == '.' && bp[1] == '/') repl += 2; 
      switch (repl) {
      case 1:
	sprintf(buf, "%s%s", homedir, bp + 1);
	S_C(i, buf);
	break;
      case 2:
	sprintf(buf, "%s%s", cwdpath, bp + 1);
	S_C(i, buf);
	break;
      }      
    }
  }
  DV("cmplPN");
  /* special case: pth_rog */
  /* der outgoing path auf dem server kann nicht den allgemeinen erweiterungen
  ** unterworfen werden, da die angaben aus der lokalen umgebung fuer den server
  ** nicht zutreffen. es wird aber vorausgesetzt, dass das home-verzeichnis uber
  ** den gleichen absoluten pfad zu erreichen ist */
  if ((bp = S_(pth_rog))) {
    if (*bp != '/') {
	sprintf(buf, "%s/%s", homedir, bp);
	S_C(i, buf);
    }
    if (lastchar(bp) != '/') {
	sprintf(buf, "%s/", bp);
	S_C(i, buf);
    }      
  }
  DV("setROG");
  /* set options */
  /* die yes/no angaben aus dem config file im bitfeld in tefax_opt
  ** eintragen */
  for (i = SETABLE; i++ < FIRSTPROG;) {
    if (S_(i)) {
      if (strcmp(S_(i), S_(uni_no)) == 0) O_DEL(i);
      if (strcmp(S_(i), S_(uni_yes)) == 0) O_SET(i);
    }      
  }
  DV("setOPT");
  /* faxdirfile erstellen, falls nicht vorhanden */
  if ((isfile(S_(fl_faxdirfile))) != 1) {
    sprintf(buf, "%s > %s", S_(cl_echo), S_(fl_faxdirfile));
    shellxx(buf);
    if ((isfile(S_(fl_faxdirfile))) != 1) {
      printf("%s\n", S_(er_book));
      afc = 1;
    }
  }
  
  /* set server correct */
  /* durch die angabe von -l in der kommandozeile arbeitet das programm
  ** lokal, auch wenn ein faxserver im config file eingetragen ist */
  if ((((bp = S_(co_server))) && *bp == '\0') || O_GET(ARG_l))
    S_C(co_server, NULL);

  if (S_(co_server)) {
    /* die entsprechenden eintraege fuer den server pruefen*/
    sprintf(buf, "%s %s \"%s %s >> %s && %s %s %s $(%s %s 2>> %s | %s %s)\" | %s ...x..x..x >> %s", 
	    S_(cl_rshell), S_(co_server), 
	    S_(cl_type), S_(avl_fax), S_(fl_logfile), 
	    S_(cl_ls), S_(cl_lso), S_(cl_lsd), S_(cl_type), S_(avl_fax), S_(fl_logfile),
	    S_(cl_cut), S_(cl_cuto),
	    S_(cl_grep), S_(fl_logfile));
    if (shellex(buf) == 0) {
      O_SET(avl_fax);
      sok = 1;
    }
    
    sprintf(buf, "%s %s \"%s %s >> %s && %s %s %s $(%s %s 2>> %s | %s %s)\" | %s ...x..x..x >> %s", 
	    S_(cl_rshell), S_(co_server), 
	    S_(cl_type), S_(avl_sendfax), S_(fl_logfile), 
	    S_(cl_ls), S_(cl_lso), S_(cl_lsd), S_(cl_type), S_(avl_sendfax), S_(fl_logfile),
	    S_(cl_cut), S_(cl_cuto),
	    S_(cl_grep), S_(fl_logfile));
    if (shellex(buf) == 0) {
      O_SET(avl_sendfax);
      sok = 1;
    }
   
    /* das ausgangsverzeichnis wird nur auf die rechte des eigentuemers ueberprueft */
    if (strlen(S_(pth_rog))) {
      sprintf(buf, "%s %s %s %s %s %s 2>> %s | %s drw >> %s", 
	      S_(cl_rshell), S_(co_server), 
	      S_(cl_ls), S_(cl_lso), S_(cl_lsd), S_(pth_rog), S_(fl_logfile),
	      S_(cl_grep), S_(fl_logfile));
      if (shellex(buf) != 0) {
	printf("%s\n%s\n%s %s %s", S_(msg_store), S_(msg_setup),
	       S_(pth_rog), S_(uni_on), S_(co_server));
	afc = 1;
      }
    }
  }
  DV("chkSERV");
  /* program check */
  /* ueberpruefung, ob die benoetigten shell-programme verfuegbar sind. insbesondere
  ** wenn kein config file gefunden wird und das programm mit den voreinstellungen
  ** arbeitet */

  /* die programme zum senden muessen fuer den programmcheck von ihren kommando-
  ** zeilenparametern befreit werden. dazu werden die ersten zeichen in co_send
  ** bzw. in co_send_f in die entsprechenden avl_xxx strings kopiert */
  if (S_(co_send) && strlen(S_(co_send)))
    sscanf(S_(co_send), "%s", buf);
  else buf[0] = '\0';
  S_C(avl_sendfax, buf);
  S(sendfax program, S_(avl_sendfax));

  if (S_(co_send_f) && strlen(S_(co_send_f)))
    sscanf(S_(co_send_f), "%s", buf);
  else buf[0] = '\0';
  S_C(avl_fax, buf);
  S(fax program, S_(avl_fax));

  bp = NULL; i = 0;
  if (O_GET(wo_p_check))

    /* der erste versuch, die programme zu finden, basiert auf den unveraenderten
    ** angaben aus dem config file oder den voreinstellungen, damit koennen auch
    ** programme verwendet werden, die sich nicht im suchpfad befinden */
    do {
      if (bp) {
	sprintf(buf, "%s/", bp);
	i = strlen(buf);
      }
      for (k = FIRSTPROG; ++k < LASTPROG;) {
	if ((!O_GET(k)) && (S_(k)) && (strlen(S_(k)) > 0)) {
	  strcpy(buf + i, S_(k));
	  /*
	    sscanf(S_(k), "%s", buf + i);
	    */
	  
	  if (stat(buf, &stbuf) == 0)
	    
	    /* die programme gs und g3vga muessen nicht nur auf ausfuehrbarkeit
	    ** sondern auch auf das gesetzte bit SUID ueberprueft werden, da sie
	    ** auf die svga librarys zugreifen und sonst der zugriff auf den 
	    ** bildschirm verweigert wird */
	    if ((k < avl_fax && ((stbuf.st_mode) & S_ISUID) == S_ISUID) ||
		/* falls ein faxserver verwendet wird, duerfen die sendeprogramme 
		** (sendfax bzw. das faxscript) lokal nicht ueberpueft werden */
		(k == avl_sendfax && (!S_(co_server)) && ((stbuf.st_mode) & S_IXOTH) == S_IXOTH) || 
		(k == avl_fax     && (!S_(co_server)) && ((stbuf.st_mode) & S_IXOTH) == S_IXOTH) || 
		/* die anderen programme werden einfach auf ausfuerbarkeit ueberprueft */
		(k > avl_fax && ((stbuf.st_mode) & S_IXOTH) == S_IXOTH)) O_SET(k);
	}
      }
      
      /* programme die mit den vorgaben nicht gefunden wurden, werden mit
      ** den angaben aus der PATH variable erweitert und erneut gesucht */
    } while ((bp = strtok(bp ? NULL : search_path, ":")));
  else 
    for (k = FIRSTPROG; ++k < LASTPROG;) O_SET(k);
    
  DV("chkPROG");
  /* dir structure */
  /* fuer die temporaeren dateien sowie die sicherheitskopien verwndet das 
  ** programm eine verzeichnisstruktur, die angelegt wird, falls einzelne
  ** verzeichnisse nicht gefunden werden */
  for (i = FIRSTPATH; i++ < PATHCONTROL;)
    if (S_(i) && isdir(S_(i)) != 1) {
      sprintf(buf, "%s %s", S_(cl_mkdir), S_(i));
      buf[strlen(buf) - 1] = '\0';
      shellxx(buf);
    }

  /* must test success */
  /* fuer den fall, dass der schreibzugriff bei der erstellung der verzeichnisse
  ** verweigert wurde, werden die angaben noch einmal ueberprueft */
  /* ausserdem sollte der schreibzugriff IN den verzeichnissen getestet werden */
  if (isdir(S_(pth_my)) && isdir(S_(pth_log)) && isdir(S_(pth_del)) &&
      isdir(S_(pth_ps)) && isdir(S_(pth_pgm)) && isdir(S_(pth_ric)) != 1) {
    /* error msg !!!! */
    if (!isdir(S_(pth_my ))) printf("%s\n", S_(msg_store)); 
    if (!isdir(S_(pth_log))) printf("%s\n", S_(msg_log)); 
    if (!isdir(S_(pth_del))) printf("%s\n", S_(msg_del)); 
    if (!isdir(S_(pth_ps ))) printf("%s\n", S_(msg_ps)); 
    if (!isdir(S_(pth_pgm))) printf("%s\n", S_(msg_pgm)); 
    if (!isdir(S_(pth_ric))) printf("%s\n", S_(msg_g3)); 
    printf("%s\n", S_(msg_setup)); 
    printf("\t%s\n", S_(pth_my)); 
    getchar();
    exit(1);
  }
  DV("setDIR");
  /* das programm ruft zu senden immer das programm in co_send (also die angaben
  ** fuer mgetty+sendfax) auf. wenn efax verfuegbar ist, werden diese angaben mit
  ** den eintraegen fuer efax einfach ueberschrieben */
  if (O_GET(avl_fax)) {
    S_C(co_send, S_(co_send_f));
    S_C(co_low, S_(co_low_f));
    S_C(fm_readfaxdir, S_(fm_readfaxdir_f));
    S_C(fm_writefaxdir, S_(fm_writefaxdir_f));
    sok = 1;
  }    
  DV("setSEND");
  /* kontrolle, ob zumindest eine arbeitsfaehige kombination von programmen
  ** zu behandlung der fax files verfuegbar ist. fuer avl_fax wurde sok bereits
  ** oben gesetzt */
  if (O_GET(avl_sendfax)) sok = 1;
  if (O_GET(avl_efix)) pok = 1;
  if (O_GET(avl_efix) && O_GET(avl_gs)) vok = 1;
  if (O_GET(ARG_x)) {
    if ((O_GET(avl_efix) && O_GET(avl_xv)) || O_GET(avl_g3view)) vok = 1;
  }
  else if (O_GET(avl_g3vga)) vok = 1;
  
  if (sok + pok + vok == 0) {
    printf("%s\n", "ooops, that's not environment. giving up! :-(");
    getchar();
    exit(1);
  }

  /* keine moeglichkeit faxe zu senden */
  if (sok == 0) {

    /* this is the saver format for faxdir */
    S_C(fm_readfaxdir, S_(fm_readfaxdir_f));
    S_C(fm_writefaxdir, S_(fm_writefaxdir_f));
    printf("%s\n", "hmm, there is no way to send faxes");
    afc = 1;
  }

  /* keine moeglichkeit faxe anzuzeigen */
  if (vok == 0) {
    printf("%s\n", "rrg, there is no way to show faxes");
    afc = 1;
  }

  /* keine moeglichkeit faxe zu drucken */
  if (pok == 0) {
    printf("%s\n", "ohh, there is no way to print faxes");
    afc = 1;
  }
  if (afc) getchar();
  DV("endcfg");
  free(search_path); search_path = NULL;
  CE;

}
  

/* option zu fehlerbeseitigung 
*/
void printallsettings(void)
{
  int i = 0;

  CF(printallsettings);

  while (i++ < LOCKED)
    if (S_(i))
       printf("%-20.20s: %s\n", S_P(i), S_(i));

  for (i = SETABLE; i++ < LASTPROG;) {
    if (O_GET(i)) printf("%s set for %s\n", S_(i), S_P(i));
    /*    if (S_(i)) printf("%s set for %s\n", S_(i), S_P(i));*/
    else printf("NOP set for %s\n", S_P(i));
  }

  printf("%s\n", "now the system settings");
  if (O_GET(ARG_n)) printf("%s\n", "don't call system");		/* no syscalls */
  else       printf("%s\n", "really call system");
  if (O_GET(ARG_x)) printf("%s\n", "run in this console");		/* X */
  else       printf("%s\n", "start x-console if possible");
  if (O_GET(ARG_S)) printf("%s\n", "print settings and exit");		/* print opts */
  else       printf("%s\n", "don't print out variable settings");
  if (O_GET(ARG_L)) printf("%s\n", "fax res set is low");		/* resolution LOW*/
  else       printf("%s\n", "fax res set is high");
  if (S_(co_server) && !O_GET(ARG_l)) printf("%s %s\n", "get fax files from", S_(co_server));
  else       printf("%s\n", "run local");


  printf("%s", "i hope the settings are correct.\n");
  CE;
}


