/**************************************************************************

    tefax  -- front end to handle FAXes with efax or mgetty&sendfax
              and certain programms for view and print

    Copyright (C) 1997  Kai Uwe Tempel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    please send any bug reports, suggestions, greetings etc. to
	temu@server3.medizin.uni-leipzig.de

    or contact me in
	Kai Uwe Tempel, Gaertnerstr. 171, 04209 Leipzig, Germany

    call or fax: +49 341 4126014

***************************************************************************/
/****************************************************************************

tefax select name functions

***************************************************************************/

/*LINTLIBRARY */

#include <stdio.h>
#include <curses.h>
#include "tefaxdef.h"
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "debug.h"
#include "hdw.h"
#include "hul.h"
#include "hcs.h"
#include "hnc.h"
#include "hrc.h"
#include "hsp.h"
#include "hlf.h"

/******************************************************************************

preset the for_send_list

******************************************************************************/
/* 
** voreinstellung von empfaenger und faxnummer in der for_send_list aus den
** absenderdaten des selektierten fax sowie ggf. den faxdateien bei forward
** das selektierte fax steht in destination, in server und pfad die aktuelle
** faxbox (eingangsverzeichenis). select enthelt die vorschrift zur datei-
** auswahl, weil beim loeschen von faxen auch die hilfdatei (z-....) mit
** erfasst werden muss
*/

int preset_fsl(char *server, char *pfad, 
	       struct unilist *forsend, struct unilist *destination, 
	       int (*select)(const struct unilist *p, const struct unilist *q))
{
  struct unilist *flist = getuninode();

  /* die dateinamen (nicht fax-loeschen) sind gleichlautend und unterscheiden
  ** sich nur durch die erweiterung, die die seiten zaehlt und wird daher auf-
  ** steigend sortiert */
  int (*sortby)(const void *, const void *) = comp_name;

  CF(preset_fsl);
  if (!pfad || !forsend || !destination) {CE; return 0;}

  /* speicher fuer die felder in der wurzel der neuen liste bereitstellen */
  flist->name = (char *) malloc(BUFSIZ);
  flist->full = (char *) malloc(BUFSIZ);

  /* falls der name des absenders bekannt ist, uebernehmen */
  if (destination->name) strcpy(flist->name, destination->name);
  else flist->name[0] = '\0';

  /* quellpfad fuer die faxdateien eintragen */
  strcpy(flist->full, pfad);

  /* quellverzeichnis einlesen und die mit select ausgewaehlten dateien in die 
  ** for_send_list uebernehmen */
  if (getdire(server, NULL, flist)) {
    unisort(flist, sortby);
    uniselect(flist, select);
    movenodes(flist, forsend, select_all);
    uniselect(forsend, select_all);
    countsel(forsend);
  }
  
  freeunilist(flist); flist = NULL;
  CE;
  return forsend->cnt;
}

/*
** listet die directories der verzeichnisstruktur und haengt ausgewaehlte dateien
** an die forsendliste an
*/
int selectfile(char *server, char *pfad, struct unilist *forsend, char *helpline)
{
  int ck = 0;
  struct unilist *flist = getuninode();

  /* ausgabeformat fuer die dateiliste auf dem bildschirm */
  char *(*format_print)(struct unilist *) = pn_file;

  /* funktion zum sortieren der dateiliste. soll sich mal im config file 
  ** einstellen lassen */
  int (*sortby)(const void *, const void *) = comp_name;

  /* long wegen berechnung aus zeiger
  ** x[oi] zeigt auf die abschliessende null, zeiger auf das letzte zeichen 
  ** waere eleganter, del dann einfach buf[oi] = '\0' */
  long oi = 0;

  /* das zuletzt gelesene eingangs- bzw das ausgangsverzeichnis werden abhaengig
  ** von der aufrufenden funktion gespeichert */
  static char oldic[BUFSIZ] = "";
  static char oldog[BUFSIZ] = "";

  char buf[BUFSIZ];

  /* der laengste bei allen selektierten dateien gleiche string */
  char *maxeq = NULL;

  /* das eingabefenster */
  static WINDOW *wfile = NULL;

  int newscan = 0;
  int moved = 0;

  CF(selectfile);
  pacline(line_st, S_(sts_file));

  if (!forsend) { CE; return 0; }

  /* speicher fuer die wurzel der neuen flist bereitstellen und initialisieren */
  flist->name = (char *) malloc(BUFSIZ);
  flist->full = (char *) malloc(BUFSIZ);
  flist->name[0] = '\0';
  flist->full[0] = '\0';

  /* bei den eingeschalteten optionen wo_mem_ic und wo_mem_og wird das gespeicherte
  ** eingangs- (oldic) bzw. ausgangsverzeichnis (oldog) als pfad voreingestellt */
  if (helpline == S_(hl_send_f)) {
    if (O_GET(wo_mem_og) && strlen(oldog)) strcpy(flist->full, oldog);}
  else {
    if (O_GET(wo_mem_ic) && strlen(oldic)) strcpy(flist->full, oldic);}
  if (flist->full[0] == '\0') strcpy(flist->full, pfad);

  /* verzeichnis einlesen, sortieren und anzeigen */
  strcpy(flist->full, getdire(server, NULL, flist));
  unisort(flist, sortby);
  uniselect(flist, select_all);
  printinpad(flist, newlist, format_print);

  /* select_file hat eine eigene menuebeschreibung (zeilen c1 und c2) */
  pacline(line_c1, S_(mnu_1fn));
  pacline(line_c2, S_(mnu_2f));
  
  /*  verzeichnisnamen im eingabefenster eintragen und anzeigen */
  sprintf(buf, "%s%s", helpline, cutonbeg(flist->full, 78 - strlen(helpline)));
  wfile = printaline(line_af, buf);

  cbreak();    /* funktionstasten erlauben, zum scrollen der liste */
#ifndef NONC
  noecho();    /* die ausgabe machen wir selbst */
#endif    
  for (
#ifndef NONC
       oi = strlen(flist->full); (ck = wgetch(wfile)) != '\n'; /* schleife bis enter */
#else
       oi = 0; (ck = getchar()) != 'q'; /* beim debuggen mit dem gdb muss die eingabe
					** eines jeden buchstaben mit enter abgeschlossen 
					** werden. daher abbruch der schleife mit q */
#endif    
       ) {
    switch(ck) {
    case '\n':                          /* enter beim debuggen ausschalten */
      continue;
    case KEY_UP:
      printinpad(flist, move_up, format_print);
      moved = 1;
      break;
    case KEY_DOWN: 
      printinpad(flist, move_down, format_print);
      moved = 1;
      break;
    case KEY_LEFT: 
      break;                            /* nix */
    case KEY_RIGHT: 
      break;                            /* nix */
    case 0x1B:
      freeunilist(flist); flist = NULL;
      CE;
      return 0;
      break;			        /* abbruch (ESC) */
    case '\t':                          /* tabulator vervollstaendigt soweit als moeglich */
      
      /* laengsten allen eintraegen gemeinsamen string finden */
      if (flist->nextsel && (maxeq = findmaxeq(flist))) {

	/* oi zeigt auf das stringende (die abschliessende null) des strings vor dem aktuellen
	** switch-durchlauf. damit kann ein string auch nach der erweiterung mit TAB um eine 
	** unbekannte anzahl von zeichen mit del wieder auf die alte laenge gekuerzt werden */
	oi = strrchr(flist->full, '/') + 2 - flist->full + strlen(flist->name);

	/* mit ->name werden die eintraege selektiert, daher darf dieser string das asterisk
	** am stringende nicht enthalten. aus diesem grunde die parallele verwendung von
	** ->name und ->full mit dem dateinamen und dem absoluten dateinamen */
	strcpy(flist->name, maxeq);

	/* erweiterung des vollen namen */
	strcpy(strrchr(flist->full, '/') + 1, maxeq);

	/* bei mehrfachen druecken von TAB zeigt oi ein zeichen zu weit nach hinten */
	if (strlen(flist->full) < oi) oi = strlen(flist->full);

	/* wenn durch die erweiterung ein verzeichnisname entstanden ist, slash anhaengen
	** und einlesen des verzeichnisses initiieren */
	if (lastchar(flist->full) == '/') newscan = 1;
      }
      break;

      /* backspace und delete kuerzen den string anhand von oi auf die alte laenge,
      ** normalerweise um ein zeichen, nach der TAB-erweiterung jedoch mehr. wenn ein 
      ** slash geloescht wird, muss das uebergordnete verzeichnis eingelesen werden */
    case KEY_DC:
      if (oi > 1) {
	if (flist->full[oi - 1] == '/') newscan = 1;
	flist->full[--oi] = '\0';
	strcpy(flist->name, pfnf(flist->full));
      }
      else moved = 1;
      break;
    case KEY_BACKSPACE:
      if (oi > 1) {
	if (flist->full[oi - 1] == '/') newscan = 1;
	flist->full[--oi] = '\0';
	strcpy(flist->name, pfnf(flist->full));
      }
      else moved = 1;
      break;

      /* das anhaengen eines asterisk (*) selektiert alle noch angezeigten dateien */
    case MK_STAR:

      /* oi anpassen, damit bei einem folgenden delete nur das asterik geloescht wird */
      oi = strlen(flist->full);

      /* so tun, als wenn kein zeichen eingegeben worden waere */
      moved = 1;

      /* nicht zwei asterik !! */
      if (flist->full[oi - 1] == '*') continue;

      flist->full[oi++] = ck;         /* asterisk anhaengen */
      flist->full[oi] = '\0';         /* string komplettieren */
      break;
    default:
      oi = strlen(flist->full);
      if (!isgraph(ck) || !flist->nextsel) continue;

      /* asterisk immer letztes zeichen */
      if (flist->full[oi - 1] == '*') continue;

      /* verzeichnisse muessen eingelesen werden */
      if (ck == '/') newscan = 1;

      flist->full[oi++] = ck;         /* falls buchstabe, anhaengen */
      flist->full[oi] = '\0';         /* string komplettieren */

      /* angabe in ->name angleichen */
      strcpy(flist->name, pfnf(flist->full));
    }
    
    if (newscan) {
      strcpy(flist->full, getdire(server, NULL, flist));
      oi = strlen(flist->full);
      unisort(flist, sortby);
      flist->name[0] = '\0';
      newscan = 0;
    }
    /* nach der eingabe eines zeichen die dateien fuer die anzeige neu auswaehlen und
    ** anzeigen bzw. fehlermeldung, wenn keine datei auf den eingaestring passt */
    if (!moved) {
      if (uniselect(flist, select_tor))
	printinpad(flist, newlist, format_print);
      else
	printinerr(S_(ind_file), flist->full);
    }
    else moved = 0;
    
    /************ eingabezeile: scroll am rechten rand;  *****/
    sprintf(buf, "%s%s", helpline, cutonbeg(flist->full, 78 - strlen(helpline)));
    wfile = printaline(line_af, buf);
  }

  /*************************************************************
  ** ende switch
  *************************************************************/

  /* ausgewaelte datei(en) in die for_send_list uebernehmen */
  if (flist->nextsel) {

    /* bei send fax (helpline) werden alle dateien (aber keine verzeichnisse)
    ** uebernommen, sonst (forward, view, print) nur faxdateien */
    int (* checksel)(const struct unilist *p, const struct unilist *q) =
      (helpline == S_(hl_send_f)) ? select_nod : select_g3;

    /* wenn kein asterisk am ende wird nur der erste angezeigte eintrag uebernommen */
    if (lastchar(flist->full) != '*') {
      flist->nextsel->nextsel = NULL;
      movenodes(flist, forsend, checksel);
    }
    /* mit asterisk alle ueber die nextsel-zeiger erreichbaren eintraege */
    else movenodes(flist, forsend, checksel);

    uniselect(forsend, select_all);
    countsel(forsend);
  
    /* gelesenens verzeichnis merken */
    if (helpline == S_(hl_send_f)) strcpy(oldog, pfnp(flist->full));
    else                           strcpy(oldic, pfnp(flist->full));
  }

  /* temp. liste wieder freigeben */
  freeunilist(flist); flist = NULL;
  printaline(line_af, "\n");
  printaline(line_st, "\n");

  CE;
  return forsend->cnt;
}

/* prepare_fsl stellt ein menue zur verfuegung, welches bis zum abbruch select_file
** aufruft und aus den mit dieser funktion ausgewaehlten dateien eien liste zusammen-
** stellt, die schliesslich den funktionen aus mm.c uebergeben werden kann. die 
** wurzel der liste enthaelt bereits alle fuer die weitere verarbeitung noetigen
** daten, wie den empfaenger, faxnummer oder verzeichnisangaben
** der erste aufruf von select_file erfolgt mit den angaben in server und pfad, bevor
** das menue das erste mal auf dem bildschirm erscheint. dieser erste aufruf von 
** select_file wird unterdrueckt, wenn die forsend_liste bereits eintraege von
** preset_fsl enthaelt
*/
int prepare_fsl(char *server, char *pfad, struct unilist *forsend, char *helpline)
{
  static WINDOW *wask;
  char *(*format_print)(struct unilist *) = pn_full;

  char *ernosel = S_(ind_nofsel);
  char *askline = S_(sts_ask);

  int cposy, cposx, ck;
  char fsname[BUFSIZ] = "";
  char fsfull[BUFSIZ] = "";
  char fsdesc[BUFSIZ] = "";
  int moved = newlist;
  
  CF(prepare_fsl);
  cbreak();
  
  /* bei leerer liste (forsend) wird vor dem menue selectfile aufgefufen, damit
  ** keine leere liste zur weiteren bearbeitung angeboten wird */
  if (!forsend->nextsel) {
    selectfile(server, pfad, forsend, helpline);
  }
  /* kontrolle, ob selectfile erfolgreich war, ansonsten return 0 */
  if (!forsend->nextsel) {
    cleanunilist(forsend);
    CE;
    return 0;
  }
  /* kompakte schreibweise: ueberprueft nur die reckgabe von selectfile:

  if (!forsend->nextsel && (!selectfile(server, pfad, forsend, helpline))) {
    cleanunilist(forsend);
    CE;
    return 0;
  }

   */

  /* um die daten im wurzelknoten von forsend auch ueber die aufrufe von,
  ** werden die strings in buffern gesichert. */
  if (forsend->name) strcpy(fsname, forsend->name);
  if (forsend->full) strcpy(fsfull, forsend->full);
  if (forsend->desc) strcpy(fsdesc, forsend->desc);

  /* liste anzeigen */
  printinsel(forsend, newlist, format_print);

  pacline(line_c1, S_(mnu_1s));
  pacline(line_c2, S_(mnu_2s));

  /* in der 'kommadozeile' wird die eingabe abgefragt, die voreinstellung
  ** send ist in askline enthalten und die abfrage getyx wird benutzt, um
  ** den cursor auf das letzte zeichen (das s) von askline zu setzen */
  wask = printaline(line_co, askline);
  getyx(wask, cposy, cposx);

  while (
#ifndef NONC
	 (ck = mvwgetch(wask, cposy, cposx - 1))
#else
	 (ck = getchar())
#endif
	 ) {  /* hauptmenue, bis (q)uit */
    chkul(forsend, "in switch of prepare_fsl");
    switch (ck) {
    case MK_FORGET:
      printaline(line_co, "\n");
      cleanunilist(forsend);
      CE;
      return forsend->cnt;	/* return 0 */

      /* MK_SEND_ED und return bewirken das gleiche, es sei denn
      ** die forsendliste ist leer. 
      ** die gesicherten strings werden wieder in die wurzel von
      ** forsend geschrieben und die funktion beendet. */
    case MK_SEND_ED:
      free(forsend->name); forsend->name = NULL;
      if (strlen(fsname)) forsend->name = strdup(fsname);
      free(forsend->full); forsend->full = NULL;
      if (strlen(fsfull)) forsend->full = strdup(fsfull);
      free(forsend->desc); forsend->desc = NULL;
      if (strlen(fsdesc)) forsend->desc = strdup(fsdesc);
      printaline(line_co, "\n");
      CE;
      return forsend->cnt;
#ifndef NONC
    case '\n':
      if (askline == S_(sts_ask)) {
	free(forsend->name); forsend->name = NULL;
	if (strlen(fsname)) forsend->name = strdup(fsname);
	free(forsend->full); forsend->full = NULL;
	if (strlen(fsfull)) forsend->full = strdup(fsfull);
	free(forsend->desc); forsend->desc = NULL;
	if (strlen(fsdesc)) forsend->desc = strdup(fsdesc);
	printaline(line_co, "\n");
	CE;
	return forsend->cnt;
      }
      else {
	/* eine leere liste laesst sich nicht verschicken oder
	** anzeigen. deshalb wird die voreinstellung von send in
	** append geaendert */
	waddstr(wask, S_(mna_a_file));      
	selectfile(server, pfad, forsend, helpline);
	pacline(line_c1, S_(mnu_1s));
      }
      break;
#endif
    case MK_NEW_LIST:
      /* new list loescht alle eintaege aus der liste, und bietet
      ** dann den neuaufbau an */
      waddstr(wask, S_(mna_a_file));      
      cleanunilist(forsend);
      selectfile(server, pfad, forsend, helpline);
      pacline(line_c1, S_(mnu_1s));
      break;
    case MK_DOWN:
      if (forsend->nextsel && forsend->cnt > 1)
	moved = move_down;
      break;
    case MK_UP:
      if (forsend->nextsel && forsend->cnt > 1)
	moved = move_up;
      break;
    case KEY_DOWN:
      if (forsend->nextsel && forsend->cnt > 1)
	moved = move_down;
      break;
    case KEY_UP:
      if (forsend->nextsel && forsend->cnt > 1)
	moved = move_up;
      break;
    case MK_APPEND:
      /* mit selectfile eine oder mehrere dateinamen an die liste
      ** anhaengen. das anhaengen geschieht immer am ende der liste */
      waddstr(wask, S_(mna_a_file));      
      selectfile(server, pfad, forsend, helpline);
      pacline(line_c1, S_(mnu_1s));
      break;
    case MK_REMOVE:
      /* einen einzelnen eintrag aus der liste loeschen. es wird immer
      ** der eintrag entfernt, auf dem sich gerade der balkencursor 
      ** befindet */
      waddstr(wask, S_(mna_d_file));            
      if (forsend->nextsel) remuninode(forsend, forsend->nextsel);
      break;
    } /* end switch */

    /* anzeige fuer die naechste eingabeaufforderung vorbereiten:
    ** liste oder meldung leere liste anzeigen, status ausgeben und 
    ** die menuezeile mit der voreinstellung fuer die eingabe auswaehlen */
    if (forsend->nextsel) {
      printinsel(forsend, moved, format_print);
      pacline(line_c2, S_(mnu_2s));
      askline = S_(sts_ask);
    }
    else {
      printinerr(S_(ind_tare), ernosel);
      pacline(line_c2, S_(mnu_2s0));
      askline = S_(sts_ask0);
    }
    moved = newlist;

    /* cursor wieder im eingabefenster auf das letzte zeichen setzen, 
    ** welches die voreinstellung enthaelt */
    wask = printaline(line_co, askline);
    getyx(wask, cposy, cposx);
  }
  CE;
  return forsend->cnt;
}










