/**************************************************************************

    tefax  -- front end to handle FAXes with efax or mgetty&sendfax
              and certain programms for view and print

    Copyright (C) 1997  Kai Uwe Tempel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    please send any bug reports, suggestions, greetings etc. to
	temu@server3.medizin.uni-leipzig.de

    or contact me in
	Kai Uwe Tempel, Gaertnerstr. 171, 04209 Leipzig, Germany

    call or fax: +49 341 4126014

***************************************************************************/
/****************************************************************************

tefax select name functions

***************************************************************************/

/*LINTLIBRARY */

#include <stdio.h>
#include <curses.h>
#include "tefaxdef.h"
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "hdw.h"
#include "hul.h"
#include "hcs.h"
#include "hnc.h"
#include "hrc.h"
#include "hsp.h"
#include "hlf.h"

/******************************************************************************

selectname liest die telefonbuchdatei ein und waehlt aus den eintraegen einen
namen sowie die dazugehoerige faxnummer aus und traegt sie in den wurzelknoten
von forsend ein. in faxdir wird der name der telefonbuchdatei uebergeben, in 
helpto die eingabe-menue-zeile, damit beim aufruf durch name-delete nur 
eingetragene namen akzeptiert werden. andernfalls wird der benutzer nach der 
eingabe eines noch nicht eingetragenen empfaengers zur eingabe der faxnummer
aufgefordert. nach der eingabe eines eingetragenen empfaengers wird die
bekannte faxnummer zur bestaetigung oder edition eingeblendet. die erweiterung
der namen erfolgt ebenso wie bei selectfile (auch mit TAB)

******************************************************************************/

int selectname(char *faxdir, struct unilist *forsend, char *helpto)
{
  int ck = 0;
  struct unilist *nlist = getuninode();
  char *helpno = S_(hl_call_no);
  char *server = S_(co_server);
  char *faxdir_fmt = S_(fm_readfaxdir);
  char *write_fmt = S_(fm_writefaxdir);
  int usefax = 0;
  char *(*format_print)(struct unilist *) = pn_name;
  int (*sortby)(const void *, const void *) = comp_name;
  int oi = 0;
  char buf[BUFSIZ];
  char *maxeq = NULL;
  static WINDOW *wempfaenger = NULL;
  static WINDOW *wnummer = NULL;
  WINDOW *wask;
  int ppreset = 0, delname = 0;
  char *m, *s;
  int moved = newlist;
  int newname = 1, llent = 0;	/* anzahl der angezeigten eintraege in der 
				** namensliste jetzt und zuletzt: 
				** last_listet_entries 
				** newname ist nullaktiv und bedeuted die eingabe
				** eines neuen namen */

  CF(selectname);

  /* erfassen des formates der telefonbuchdatei und der aufrufenden funktion
  ** bei hl_del_no werden keine unbekannten namen akzeptiert
  ** ppreset uebernimmt den empfaenger aus dem wurzelknoten von forsend falls
  ** verwertbare eintraege gefunden werden, unterwirfst sie aber trotzdem einer
  ** bestaetigung durch den user */
  if (O_GET(avl_fax)) usefax = 1;
  if (helpto == S_(hl_del_no)) delname = 1;
  if (O_GET(wo_preset_no)) ppreset = 1;

  /* wenn die wurzel von forsend einen empfaengernamen enthaelt und wo_preset_no
  ** gesetzt ist, werden die daten in die wurzel der namensliste uebernommen. ist
  ** kein name angegeben, wird versucht, die faxnummer zu uebernehmen */
  nlist->name = (char *) malloc(BUFSIZ);
  nlist->name[0] = '\0';
  if (O_GET(wo_preset_no)) {
    if (forsend->name)
      nlist->name = strcpy(nlist->name, forsend->name);
    else if (forsend->desc)
      nlist->name = strcpy(nlist->name, forsend->desc);
  }
  
  /* telefonbuchdatei einlesen, alphabetisch sortieren und ausgeben */
  scanfaxdir(server, faxdir, nlist, faxdir_fmt);
  unisort(nlist, sortby);
  uniselect(nlist, select_tor);
  printinpad(nlist, newlist, format_print);

  /* auch die menuezeilen schoen setzen */
  pacline(line_c1, S_(mnu_1fn));
  pacline(line_c2, S_(mnu_2n));
  pacline(line_st, S_(sts_name));



  sprintf(buf, "%s%s", helpto, cutonbeg(nlist->name, 78 - strlen(helpto)));
  wempfaenger = printaline(line_an, buf);
  cbreak();    /* funktionstasten erlauben, zum scrollen der liste */
#ifndef NONC
  noecho();    /* die ausgabe machen wir selbst */
  for (oi = 0; (ck = wgetch(wempfaenger)) != '\n';) /* schleife bis enter */
#else
  printf("\n%s\n", buf);
  for (oi = 0; (ck = getchar()) != 'q';) /* beim debuggen mit dem gdb muss die eingabe
					 ** eines jeden buchstaben mit enter abgeschlossen 
					 ** werden. daher abbruch der schleife mit q */ 
#endif    
    {
    switch(ck) {
    case '\n':                           /* enter beim debuggen ausschalten */
      continue;
    case KEY_UP:
      moved = move_up;
      break;
    case KEY_DOWN: 
      moved = move_down;
      break;
    case KEY_LEFT: 
      break;                             /* nix */
    case KEY_RIGHT: 
      break;                             /* nix */
    case 0x1B:
      freeunilist(nlist); nlist = NULL;
      CE; return 0;
      break;                             /* abbruch (ESC) */
    case '\t':                           /* tabulator vervollstaendigt soweit als moeglich */

      /* der aufruf von findmaxeq ist hier wesentlich einfacher, als in selectfile, da zum
      ** einen keine pfadnamen verwaltet werden muessen, und zum anderen die auswahl von 
      ** mehreren name via asterisk nicht noetig ist. der string, den findmaxeq zurueck-
      ** gibt wird einfach in die wurzel von nlist uebernommen */
      if ((maxeq = findmaxeq(nlist))) {
	oi = strlen(nlist->name);
	strcpy(nlist->name, maxeq);
      }
      break;

      /* backspace und delete kuerzen den string anhand von oi auf die alte laenge,
      ** normalerweise um ein zeichen, nach der TAB-erweiterung jedoch mehr. */
    case KEY_DC:
      nlist->name[oi] = '\0';
      if (oi > 0) oi--;
      break;
    case KEY_BACKSPACE:
      nlist->name[oi] = '\0';
      if (oi > 0) oi--;
      break;
    default:

      /* abhaengig vom format der telefonbuchdatei koennen bestimmte zeichen im
      ** namensstring nicht zugelassen werden. akzeptiert werden alle sichtbaren
      ** zeichen, bei der verwendung von efax ausserdem leerzeichen und tabulatoren */
      if (oi < BUFSIZ - 1 && (isgraph(ck) || ((usefax) && isprint(ck)))) {
	oi = strlen(nlist->name);
	nlist->name[oi] = ck;          /* falls buchstabe, anhaengen */
	nlist->name[oi + 1] = '\0';    /* string komplettieren */
      }
    } /* end switch */
    
    /* um bei der eingabe eines neuen empfaenger nicht bei jedem buchstaben die 
    ** anzeigeroutine aufzurufen, wird die liste nur angezeigt, wenn entweder
    ** namen anzuzeigen sind, oder die angezeigten namen durch eine leere liste
    ** ersetzt werden muessen. newname ist nullaktiv */
    llent = newname;
    if ((newname = uniselect(nlist, select_tor)) || llent)
      printinpad(nlist, moved, format_print);
    moved = newlist;

    /************ eingabezeile: scroll am rechten rand;  *****/
    sprintf(buf, "%s%s", helpto, cutonbeg(nlist->name, 78 - strlen(helpto)));
    wempfaenger = printaline(line_an, buf);
  } 

  /* end while */

  /* ersten angezeigten name auswaehlen und in die wurzel eintragen, falls der name
  ** nicht mit TAB vervollstaendigt wurde. damit ist es jedoch nicht mehr moeglich
  ** namen einzugeben, die vllstaendig in anderen namen enthalten sind */
  if (nlist->nextsel && nlist->nextsel->name)
    strcpy(nlist->name, nlist->nextsel->name);

  /* das ergebnis der auswahl anzeigen */
  sprintf(buf, "%s%s", helpto, cutonbeg(nlist->name, 78 - strlen(helpto)));
  wempfaenger = printaline(line_an, buf);
  
  /* die eingabe eines leeren string bewirkt wie ESC den abbruch der funktion 
  ** ebenso wird die funktion abgebrochen, wenn der aufruf durch name_delete
  ** erfolgte und ein unbekannter name eingegeben wurde */
  if (strlen(nlist->name) == 0) {
    freeunilist(nlist); nlist = NULL;
    CE; return 0;
  }
  if (delname) {
    if (!nlist->nextsel) {
    freeunilist(nlist); nlist = NULL;
    pacline(line_st, S_(sts_noc));
    CE; return 0;
    }
    else {			/* ask confirm !! */
      if (O_GET(wo_needconfirm)) {

	/* die frage zusammenbasteln, so dass sie vollstaendig in eine zeile passt */
	int cposy = 0, cposx = 0;
	sprintf(buf, "%s %s %s %s", S_(ask_1), S_(ask_delete), nlist->nextsel->name, S_(ask_2));
	if (strlen(buf) > 78)
	  sprintf(buf, "%s %s %s%s", S_(ask_1), S_(ask_delete), nlist->nextsel->name, S_(ask_2s));
	if (strlen(buf) > 78)
	  sprintf(buf, "%s %s %s%s", S_(ask_1), S_(ask_delete), 
		  cutonend(nlist->nextsel->name, 78 - strlen(S_(ask_delete)) - 
			   strlen(S_(ask_1)) - strlen(S_(ask_2s))), S_(ask_2s));

	/* fage ausgeben und cursor auf das letzte zeichen [y] positionieren */
	wask = pacline(line_st, buf);
	getyx(wask, cposy, cposx);
#ifndef NONC
	(ck = mvwgetch(wask, cposy, cposx - 1));
#else
	(ck = getchar());
#endif
	if (ck == '\n') ck = MK_YES; /* voreinstellung ist YES !! */
      }

      /* wenn wo_needconfirm == 0 automatische einstellung von YES */
      else ck = MK_YES;
      if (ck == MK_YES) {

	/* das eigentliche loeschen erfolgt durch updatefaxdir. der name wird in die
	** neue datei nicht uebernommen, weil nlist->desc noch auf null zeigt und dieser
	** zeiger von updatefaxdir entsprechend ausgewertet wird. das ergebnis der aktion
	** wird mit pacline ausgegeben */
	if (updatefaxdir(server, faxdir, nlist, faxdir_fmt, write_fmt) == 0)
	  pacline(line_st, S_(sts_ndel));
	else 
	  pacline(line_st, S_(er_eroc));
      }
      else pacline(line_st, S_(sts_noc));
      freeunilist(nlist); nlist = NULL;
      CE; return 1;

      /* ende der funktion, wenn der aufruf durch name_delete erfolgte */
    }
  }

				/* now the number */

  /* erst mal bildschirm (menue) reinigen */
  pacline(line_c1, "\n");
  pacline(line_c2, "\n");

  /* speicher fuer die nummer in der wurzel bereitstellen */
  nlist->desc = (char *) malloc(BUFSIZ);

  /* bei bekannem namen wird die faxnummer aus der telefonbuchdatei als voreinstellung
  ** uebernommen und im weiteren zum editieren angeboten */
  if (nlist->nextsel && nlist->nextsel->desc)
    strcpy(nlist->desc, nlist->nextsel->desc);
  else if (forsend->desc && ppreset) {

    /* aus den absenderanaben stammende faxnummern muessen einer formatanpassung
    ** unterzogen werden, damit die faxprogramme nicht durcheinander geraten 
    ** / - ( ) werden durch leerzeichen ersetzt, wenn efax verwendet wird, wobei
    ** mehrere aufeinanderfolgende leerzeichen zu einem zusammengefasst werden */
    for (m = forsend->desc, s = nlist->desc; *m != '\0'; m++)
      if (isdigit(*m) || *m == '+') {*s++ = *m; *s = '\0';}
      else if (isspace(*m)) { 
	if (usefax && !isspace(*(s - 1))) {*s++ = ' '; *s = '\0'; }
      }
      else if ((*m == '/' || *m == '-' || *m == '(' || *m == ')')) {
	if (usefax && !isspace(*(s - 1))) {*s++ = ' '; *s = '\0';}
      }
      else {
	nlist->desc[0] = '\0';
	break;
      }
  }

  /* zur not wird die voreinstellung auf einen leeren string gesetzt */
  else strcpy(nlist->desc, "");

  /* voreinstellung inclusive menuezeile ausgeben */
  sprintf(buf, "%s%s", helpno, cutonbeg(nlist->desc, 78 - strlen(helpno)));
  wnummer = printaline(line_tn, buf);

#ifndef NONC
  for (oi = 0; (ck = wgetch(wnummer)) != '\n';) /* schleife bis enter */
#else
  for (oi = 0; (ck = getchar()) != 'q';)  /* beim debuggen mit dem gdb muss die eingabe
					  ** eines jeden buchstaben mit enter abgeschlossen 
					  ** werden. daher abbruch der schleife mit q */ 
#endif    
    {
    switch(ck) {
    case '\n':                            /* enter beim debuggen ausschalten */
      continue;
    case 0x1B:                            /* abbruch mit ESC */
      pacline(line_st, S_(sts_noc));
      freeunilist(nlist); nlist = NULL;
      CE; return 0;
      break;

      /* delete und backspace loeschen einfach das letzte zeichen des strings */
    case KEY_DC:
      nlist->desc[oi] = '\0';
      if (oi > 0) oi--;
      break;
    case KEY_BACKSPACE:
      nlist->desc[oi] = '\0';
      if (oi > 0) oi--;
      break;
    default:

      /* die zulaessigen eingaben werden durch das verwendete faxprogramm bestimmt
      ** efax (usefax) laesst neben den ziffern und '+' auch space oder TAB zu */
      if (oi < BUFSIZ - 1 && (isdigit(ck) || 
			      (ck == '+' && oi == 0) ||
			      (usefax && isspace(ck)))) {
	oi = strlen(nlist->desc);
	nlist->desc[oi] = ck;         /* falls buchstabe, anhaengen */
	nlist->desc[oi + 1] = '\0';    /* string komplettieren */
      }
    }

    /* eingabe anzeigen. ggf. scroll am rechten rand */
    sprintf(buf, "%s%s", helpno, cutonbeg(nlist->desc, 78 - strlen(helpno)));
    wnummer = printaline(line_tn, buf);
    }

  /* eine faxnummer mit weniger als 3 ziffern gilt als fehleingabe und fuehrt zum
  ** abbruch der funktion, ebenso wie ein leerer string bei der namenangabe */
  if (strlen(nlist->desc) < 3) {
    freeunilist(nlist); nlist = NULL;
    CE; return 0;
  }
  
  /* eventuelle voreinstellungen in der forsendliste loeschen und dafuer die
  ** aktuellen daten eintragen */
  free(forsend->name); forsend->name = NULL;
  free(forsend->desc); forsend->desc = NULL;

  /* die if-abfrage dient nur zur sicherheit, an deser stelle MUESSEN beide
  ** variablen belegt sein */
  if (nlist->name) forsend->name = strdup(nlist->name);
  else { freeunilist(nlist); nlist = NULL; CE; return 0; }
  if (nlist->desc) forsend->desc = strdup(nlist->desc);
  else { freeunilist(nlist); nlist = NULL; CE; return 0; }
  
  /* falls ein neuer name eingegeben wurde oder die faxnummer eines bekannten 
  ** empfaengers nicht mit der angabe in der telefonbuchdatei uebereinstimmt
  ** wird updatefaxdir aufgerufen, um die telefonbuchdatei zu aktualisieren */
  if (!nlist->nextsel || (strcmp(nlist->nextsel->desc, nlist->desc) != 0)) {
    updatefaxdir(server, faxdir, nlist, faxdir_fmt, write_fmt);
  } 
  freeunilist(nlist); nlist = NULL;
  CE; return 1;
}




