/**************************************************************************

    tefax  -- front end to handle FAXes with efax or mgetty&sendfax
              and certain programms for view and print

    Copyright (C) 1997  Kai Uwe Tempel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    please send any bug reports, suggestions, greetings etc. to
	temu@server3.medizin.uni-leipzig.de

    or contact me in
	Kai Uwe Tempel, Gaertnerstr. 171, 04209 Leipzig, Germany

    call or fax: +49 341 4126014

***************************************************************************/
/****************************************************************************

tefax difinitions

***************************************************************************/
/***************************************************************************

bei der Verwendung eines Faxservers unter BSD kann das Label SERVER_IS_BSD
definiert werden. Der erste Parameter an die remoteshell lautet dann:

SERVER.LOGIN:FAX-AUSGANGSVERZEICHNIS

ansonsten wird die Adresse in folgender Form uebergeben:

LOGIN@SERVER:FAX-AUSGANGSVERZEICHNIS

Wenn in der Konfigurationsdatei aber kein Benutzername angegeben ist wird
der remoteshell ueberhaupt kein user uebergeben. Der Aufruf erfolgt dann 
lediglich mit

SERVER:FAX-AUSGANGSVERZEICHNIS

***************************************************************************/

/* #define SERVER_IS_BSD */

#ifndef MAX_PATH
#define MAX_PATH 1024
#endif

#define MAXLINE 100

#ifndef CTRL
#define CTRL(x)		((x) & 0x1f)
#endif

/* die zeilennummern der einzelnen ausgabefenster */

#define line_tl 0               /* top line           */ 
#define line_lb 2		/* list begin         */
#define line_le 12		/* list end           */
#define line_av 14		/* ueberschrift menue */
#define line_c1 15		/* 1. menuezeile      */
#define line_c2 16		/* 2. menuezeile      */
#define line_st 18		/* statuszeile        */
#define line_co 20		/* zeile command:     */
#define line_an 21		/* ask for name       */
#define line_tn 22		/* ask for number     */
#define line_af 23		/* ask for file       */

/* alle pads haben die gleiche groesse, sonst muss listlines durch   */
/* mehrere variablen ersetzt werden                                  */
/* listlines = darstellbare zeilen im fax/datei-fenster: 3-13 -> 11  */

#define listlines line_le - line_lb + 1

/* PADS                                                              */
/* alle punkteangaben im format (zeile, spalte)! -- NULLBASIERT      */
/* fenster: padoffset, begin (ul), end (dr) window                   */

#define listfenster basex,0,line_lb,0,line_le,79

/* WINDOWS                                                           */
/* size, offset (ul)                                                 */
/* in daterfenster erfolgen die Fehlermeldungen sowie die Hinweise,  */
/*  wenn die Liste leer ist                                          */

#define daterfenster listlines,80,line_lb,0

/* die neuen ncurses setzen bei getch() den stdscr in den vordergrund        */
/* so dass alle fenster verschwinden. es haette gereicht, alle eingaben in   */
/* fenstern zu machen, aber die loesung mit verschiedenen fenstern ist recht */
/* praktisch. die fenster werden in main() erstellt und menuwin_XX benannt   */

#define tl_fenster 1,80,line_tl,0
#define av_fenster 1,80,line_av,0
#define c1_fenster 1,80,line_c1,0
#define c2_fenster 1,80,line_c2,0
#define st_fenster 1,80,line_st,0
#define co_fenster 1,80,line_co,0
#define af_fenster 1,80,line_af,0
#define an_fenster 1,80,line_an,0
#define tn_fenster 1,50,line_tn,30

/* definition fuer int mv in printinsel() und printinpad() */

#define move_up -1
#define newlist 0
#define move_down 1

/* dokumentation der funktionsaufrufe. dies hilft bei der fehlersuche   */
/* darstellung in einer baumstruktur                                    */

#if defined DEBUG
#define CF(s)             logfuncs(#s)
#define CE                logfuncs(NULL)
/* alle funktionsnamen in diesem string werden nicht im logfile erfasst */
/* #define nologfunctions    "" schreibt alles mit                      */
#define nologfunctions    "pfnp pfnn pfnf pfnx isg3 cutonend"
#else
#define CF(s)             /**/
#define CE                /**/
#define nologfunctions    ""
#endif

/* checkul() ueberprueft im Prinzip nach jeder Veraenderung in den      */
/* unilisten die Konsistenz der Listenstruktur                          */
/* das ist ziemlich zeitaufwendig und nur zu empfehlen, wenn Probleme   */
/* auftreten. checkul findet trotzdem nicht alle Fehler                 */

#if defined CHKUL
#define chkul(r, m)       checkul(r , m)
#else
#define chkul(r, m)       /**/
#endif

/* key definitionen fuer die menues         */
/* SF: selected fax, ED: by invoking editor */

#define MK_DOWN        106      /* j: move down */
#define MK_UP          107	/* k: move up */
#define MK_HELP         63	/* ? */
#define MK_PRINT_SF    112	/* p */
#define MK_PRINT        80	/* P */
#define MK_VIEW_SF     118	/* v */
#define MK_VIEW         86	/* V */
#define MK_READ_SF     114	/* r */
#define MK_READ         82	/* R */
#define MK_G3_SF       103	/* g */
#define MK_G3           71	/* G */
#define MK_CALLBACK_ED  99	/* c */
#define MK_CALLBACK     67	/* C */
#define MK_SEND_ED     115	/* s */
#define MK_SEND         83	/* S */
#define MK_FORWARD_SF  102	/* f */
#define MK_FORWARD      70	/* F */
#define MK_DELETE      100	/* d: moves fax to received folder */
#define MK_REMOVE      114	/* r: removes entry in sendlist */
#define MK_NAME_DEL    110	/* n: delete a name in faxdir */
#define MK_NEW_LIST     78      /* N: clear and create a new forsendlist */
#define MK_HIGH        104	/* h: set high resolution mode */
#define MK_LOW         108	/* l: set low resolution mode */
#define MK_BOX_IN       98	/* b: switch to incoming fax box */
#define MK_BOX_DEL      66	/* B: switch to received folder */
#define MK_QUIT        113	/* q */
#define MK_FORGET      102	/* q */
#define MK_YES         121	/* y */
#define MK_NO          110	/* n */
#define MK_APPEND       97	/* a: append files to send list */
#define MK_STAR         42	/* * (asterisk) */
#define MK_SORT_N      122	/* z sort by name */
#define MK_SORT_D      117	/* u sort by desc */
#define MK_SORT_T      116	/* t sort by time */
#define MK_SORT_NR      89	/* Z sort by name */
#define MK_SORT_DR      85	/* U sort by desc */
#define MK_SORT_TR      84	/* T sort by time */
#define MK_WARR        119      /* w print warranty and gnu licence */
#define MK_SHELL        33      /* ! shellaufruf */

/* die definition einer universalen liste. in diesen knoten werden sowohl
** die daten der dateilisten, als auch der fax und namenslisten verwaltet */

typedef struct unilist {
    struct unilist *next;
    struct unilist *nextsel;
    int cnt;
    int res;
    int day;
    int month;
    int year;
    char *full; 
    char *name;
    char *desc;
} *uniptr;

