/**************************************************************************

    tefax  -- front end to handle FAXes with efax or mgetty&sendfax
              and certain programms for view and print

    Copyright (C) 1997  Kai Uwe Tempel

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    please send any bug reports, suggestions, greetings etc. to
	temu@server3.medizin.uni-leipzig.de

    or contact me in
	Kai Uwe Tempel, Gaertnerstr. 171, 04209 Leipzig, Germany

    call or fax: +49 341 4126014

***************************************************************************/
/****************************************************************************

tefax uni list funktions

*****************************************************************************
****************************************************************************/

/*LINTLIBRARY */

#include <stdio.h>
#include <string.h>		/*  */
#include <stdlib.h>
#include <ctype.h>
#include "debug.h"
#include "tefaxdef.h"		/*  */
#include "hlf.h"		/*  */
#include "his.h"
#include "hsp.h"
#include "hul.h"
#include "hcs.h"
#include <curses.h>

/***************************************************************************/

/* check_ul wird nur eingebunden, wenn DEBUG in tefaxdef definiert wurde.
** die funktion ueberprueft in einem einfachen check den inhhalt der listen.
** das S() macro schreibt das anegebene feld in die datei debug und kann 
** auf die auf die felder full und desc erweitert werden
*/

void checkul(struct unilist *node, char *mess)
{
  S(check unilist ---------- call, mess);
  if (!node) return;
  while ((node = node->next)) {
    S(cul name, node->name);
    if (node->day > 31 || node->day < 0 ||
	node->month > 12 || node->month < 0 ||
	node->year > 2000 || node->year < 0 ) {
      endwin();
      printf("\nERROR IN UNILIST: %s\n\n", mess);
      exit(1);
    }
  }
}

int compname(const void *p, const void *q)
{
  CF(compname);
  CE;
  return strcmp((*(struct unilist **)p)->name, (*(struct unilist **)q)->name);
}

int compdate(const void *p, const void *q)
{
  CF(comparr);
  CE;
  if ((*(struct unilist **)p)->year  < (*( struct unilist **)q)->year)  return -1;
  if ((*(struct unilist **)p)->year  > (*( struct unilist **)q)->year)  return  1;
  if ((*(struct unilist **)p)->month < (*( struct unilist **)q)->month) return -1;
  if ((*(struct unilist **)p)->month > (*( struct unilist **)q)->month) return  1;
  if ((*(struct unilist **)p)->day   < (*( struct unilist **)q)->day)   return -1;
  if ((*(struct unilist **)p)->day   > (*( struct unilist **)q)->day)   return  1;
  return 0;
}

int compdesc(const void *p, const void *q)
{
  CF(compdesc);
  CE;
  return strcmp((*(struct unilist **)p)->desc, (*( struct unilist **)q)->desc);
}

int comp_name(const void *p, const void *q)
{
  CF(comp_name);
  CE;
  return compname(q, p);
}

int comp_date(const void *p, const void *q)
{
  CF(comp_arr);
  CE;
  return compdate(q, p);
}

int comp_desc(const void *p, const void *q)
{
  CF(comp_desc);
  CE;
  return compdesc(q, p);
}

/* 
** doublechain verkettet die liste vollstaendig (rueckwaerts)
*/
int doublechain(struct unilist *root)
{
  struct unilist *node;
  
  CF(doublechain);

  if (!root) { CE; return -1;}
  if (!root->next) { CE; return 0;}

  chkul(root, "doublechain");
  root->next->nextsel = NULL;
  for (node = root->next; (node->next); node = node->next) 
    node->next->nextsel = node;
  CE;
  return (root->res = root->cnt);
}  

/* 
** select_all gibt fuer jeden knoten eins zurueck
*/
int select_all(const struct unilist *p, const struct unilist *q)
{
  CF(select_all);
  CE;
  return 1;
}

/*
** select_nod gibt eins zurueck, wenn der string im feld name kein
** directory bezeichnet (kein slash am stringende)
*/
int select_nod(const struct unilist *p, const struct unilist *q)
{
  CF(select_nod); CE;
  if (!p || !p->name) return 0;
  if (lastchar(p->name) == '/') return 0;
  else return 1;
}

/*
** select_tor gibt eins zurueck, wenn p->name 'matcht'
*/ 
int select_tor(const struct unilist *p, const struct unilist *q)
{
  CF(select_tor); CE;
  if (!p || !q) return 0;
  if (!p->name || !q->name) return 0; 
  if (strncmp(p->name, q->name, strlen(q->name)) == 0) return 1;
  return 0;
}

/*
** select_fax gibt fuer alle dateien, die zu einem fax gehoeren 
** eins zurueck. (inclusive hilfsfile z......)
*/
int select_fax(const struct unilist *p, const struct unilist *q)
{
  CF(select_fax); CE;
  if (!p || !q) return 0;
  if (!p->name || !q->name) return 0; 
  if (strncmp(p->name, q->name, strlen(q->name)) == 0) return 1;
  if (p->name[0] == 'z')
    if (strncmp(p->name + 3, q->name, strlen(q->name)) == 0) return 1;
  return 0;
}

/* 
** selct_one gibt nur bei exakter gleichheit eins zurueck 
*/
int select_one(const struct unilist *p, const struct unilist *q)
{
  CF(select_one); CE;
  if (!p || !q) return 0;
  if (!p->name || !q->name) return 0; 
  if (strcmp(p->name, q->name) == 0) return 1;
  return 0;
}

/*
** select_g3 gibt eins zurueck, wenn p->name als g3-file erkannt wird 
*/
int select_g3(const struct unilist *p, const struct unilist *q)
{
  CF(select_g3); CE;
  return isg3(p->name);
}

/*
** uniselect verkettet die mit select ausgewaehlten knoten der liste
** ueber die zeiger nextsel (rueckwaerts)
*/
int uniselect(struct unilist *root, 
	      int (*select)(const struct unilist *, const struct unilist *))
{
  struct unilist *flist, *lsel;
  
  CF(uniselect);

  if (!root) { CE; return -1; }
  flist = root->next;
  root->res = 0;
  
  for (lsel = NULL; flist != NULL; flist = flist->next) {
    flist->nextsel = lsel;
    if (select(flist, root)) {

      /* selektierten knoten merken und mitzaehlen */
      lsel = flist;
      root->res++;
    }
  }
  root->nextsel = lsel;

  CE;
  chkul(root, "uniselect");
  return root->res;
}

/*
** selselect verkettet die zeiger nextsel so, dass alle zeiger vor dem
** selektierten knoten auf den knoten zeigen, alle andern auf null
*/
struct unilist *selselect(struct unilist *root)
{
  struct unilist *flist, *lsel;
  
  CF(uniselect);

  if (!root) { CE; return NULL; }
  flist = root->next;
  
  for (lsel = NULL; flist != NULL; flist = flist->next) {
    if (select_one(flist, root)) {
      root->nextsel = flist;
      break;
    }
  }
  countsel(root);
  CE;
  chkul(root, "selselect");
  return root->nextsel;
}

/*
** add-extension: fuer g3 an faxspool entwickelt, wird an alle g3-files die
** extension '.g3' angehaengt, damit faxspool das format dieser dateien er-
** kennen kann. diese namen werden fuer links im ausgangsverzeichnis auf
** die originaldateien verwendet. erweitert werden nur die eintraege in 
** ->name
*/
void addex(struct unilist *root)
{
  char buf[BUFSIZ] = "";
  char *xten;
  int xlen = 0;
  char *s;
  int i = 0;

  CF(addex);
  if (!root || !root->next) {CE; return;}

  for (root = root->next; root != NULL; root = root->next) {
    
    /* s und t zeigen auf die extension */
  xlen = strlen((s = xten = pfnx(root->name)));
  switch (xlen) {
  case 0: i = 0; break; /* keine extension */

  case 2:
    
    /* die files von mgetty */
    if (root->name[0] == 'f' && (root->name[1] == 'f' || root->name[1] == 'n') && 
        (root->name[11] == '.' || root->name[11] == '-') && isdigit(*s++) && isdigit(*s++)) i = 1;
    
    /* die eigenen faxe von sendfax */
    else if (xten[0] == 'g' && xten[1] == '3') i = 2;
    else i = 5;
    break;
  case 3:
    
    /* bei den beiden folgenden abfragen muss s wieder auf t, d.h. den anfang der
    ** dateinamenendung zurueckgesetzt werden */
    /* die eigenen faxe */
    if (root->name[0] == 'f' && (root->name[1] == 'a' && root->name[2] == 'x') && ((s = xten)) &&
        isdigit(*s++) && isdigit(*s++) && isdigit(*s++)) i = 3;
    
    /* die files von efax */
    else if (isdigit(*root->name) && ((s = xten)) &&
             isdigit(*s++) && isdigit(*s++) && isdigit(*s++)) i = 4;
    else i = 5;
    break;

  default: i = 5; /*alle anderen extensions */
  }


  /* bei der verwendung von sendfax muessen die faxdateien auf .g3 enden, bei efax mit
  ** drei ziffern 
  */
  if (O_GET(avl_fax)) i += 6;

  switch (i) {
  case 0:  sprintf(buf, "%s.nox", root->name);        break; /* keine xtension      */
  case 1:  sprintf(buf, "%s.g3",  root->name);        break; /* fx.x-x.01 -> .g3    */
  case 2:  sprintf(buf, "%s.g3",  root->name);        break; /* .g3 -> .g3          */
  case 3:  sprintf(buf, "%s.g3",  root->name);        break; /* fax.001 -> .g3      */
  case 4:  sprintf(buf, "%s.g3",  root->name);        break; /* 0x.001 -> .g3       */
  case 5:  sprintf(buf, "%s.%s",  root->name, xten);  break; /* irgendeine xtension */
  case 6:  sprintf(buf, "%s.nox", root->name);        break; /* keine xtension      */
  case 7:  sprintf(buf, "%s.0%s", root->name, xten);  break; /* fx.x-x.01 -> .001   */
  case 8:  sprintf(buf, "%s.001", root->name);        break; /* .g3 -> .001         */
  case 9:  sprintf(buf, "%s.%s",  root->name, xten);  break; /* fax.001 -> .001     */
  case 10: sprintf(buf, "%s.%s",  root->name, xten);  break; /* 0x.001 -> .001      */
  case 11: sprintf(buf, "%s.%s",  root->name, xten);  break; /* irgendeine xtension */
  }
  
  free(root->name);
  root->name = strdup(buf);
  }
}

/*
** unisort scheibt alle eintraege in ein array und laesst es von qsort sortieren
** das sortierte array wird anschliessend wieder in die verkettete liste geschrieben
*/
void unisort(struct unilist *root, int (*compare)(const void *, const void *))
{
  struct unilist *pa[root->cnt];
  struct unilist *ppa;  
  int i;

  CF(unisort);
  if (!root || !root->next || !root->next->next) {CE; return;}
  for (i = 0, ppa = root->next; i < root->cnt; ppa = ppa->next)
    pa[i++] = ppa;

  qsort(pa, root->cnt, sizeof(struct unilist *), compare);

  for (i = 0, ppa = root; i < root->cnt; ppa = ppa->next)
    ppa->next = pa[i++];
  ppa->next = NULL;
  CE;
  chkul(root, "unisort");
}

/*
** selsort benutzt unisort, verkettet die liste aber anschliessend wieder 
** vollstaendig in beiden richtungen
*/
void selsort(struct unilist *root, int (*compare)(const void *, const void *))
{
  CF(selsort);
  unisort(root, compare);
  doublechain(root);
  CE;
  chkul(root, "selsort");
}

/*
** getuninode stellt einen neuen knoten zur verfuegung
** und initialisiert alle felder
*/
struct unilist *getuninode(void)
{
  struct unilist *unode;

  CF(getuninode);
    if ((unode = (struct unilist *)(malloc(sizeof(struct unilist))))) {
      unode->name = NULL;
      unode->full = NULL;
      unode->desc = NULL;
      unode->next = NULL;
      unode->nextsel = NULL;
      unode->day = 0;
      unode->month = 0;
      unode->year = 0;
      unode->cnt = 0;
      unode->res = 0;
      CE;
      return unode;
    }
    CE;
    return NULL;
}

/*
** freeuninode entfernt einen knoten und gibt den speicher wieder frei
*/
void freeuninode(struct unilist *unode)
{
  CF(freeuninode);
  if (unode) {
    free(unode->name); unode->name = NULL;
    free(unode->full); unode->full = NULL;
    free(unode->desc); unode->desc = NULL;
    free(unode); unode = NULL;
  }
  CE;
}

/*
** freeunilist entfernt mit freeuninode alle knoten aus einer liste
** inclusive dem wurzelknoten selbst und loescht damit die liste voll-
** staendig
*/
void freeunilist(struct unilist *p)
{
  struct unilist *q = p;
 
  CF(freeunilist);
  chkul(q, "freeunilist");

  q = p;
  while (q != NULL) {
      q = p->next;
      freeuninode(p); 
      p = q;
  }
  CE;
}

/*
** cleanuninde loescht den inhalt eines knotens
*/
void cleanuninode(struct unilist *p)
{
  CF(cleanuninode);

  if (p != NULL) {
    p->next = NULL;
    p->nextsel = NULL;
    p->cnt = 0;
    p->res = 0;
    p->day = 0;
    p->month = 0;
    p->year = 0;
    free(p->name); p->name = NULL;
    free(p->full); p->full = NULL;
    free(p->desc); p->desc = NULL;
  }
  CE;
}
  
/*
** cleanunilist loescht den inhalt einer liste. auch der inhalt des 
** wurzelknotens wird geloescht
*/
void cleanunilist(struct unilist *p)
{
  CF(cleanunilist);

  if (p) {
    freeunilist(p->next); p->next = NULL;
    cleanuninode(p);
  }
  CE;
}
  
/*
** insertuninode fuegt einen neuen knoten mit den uebergebenen daten
** in die liste ein
*/  
struct unilist *insuninode(struct unilist *root,
		  char *name, char *full, char *desc,
		  int cnt, int res, int day, int month, int year)
{
  struct unilist *unode;

  CF(insert uninode);
  if ((unode = getuninode()) == NULL) { CE; return NULL; }
  chkul(root, "insuninode b");

  unode->next = root->next;
  unode->nextsel = NULL;

  if (name) unode->name = strdup(name);
  else unode->name = NULL;
  if (full) unode->full = strdup(full);
  else unode->full = NULL;
  if (desc) unode->desc = strdup(desc);
  else unode->desc = NULL;

  unode->cnt = cnt;
  unode->res = res;
  unode->day = day;
  unode->month = month;
  unode->year = year;
  
  root->next = unode;
  root->cnt++;
  CE;
  chkul(root, "insuninode e");
  return unode;
}

/*
** remuninode entfernt einen knoten aus der liste
** damit die nextsel-zeiger konsistent bleiben wird die liste
** mit doublechain voll in beide richtungen verkettet
*/
void remuninode(struct unilist *root, struct unilist *delnode)
{
  struct unilist *unode;

  CF(remove uninode);
  chkul(root, "remuninode");

  if (root && delnode) {
    doublechain(root);
    for (unode = root->next; unode != NULL; unode = unode->next) {
      if (unode->nextsel == delnode) unode->nextsel = delnode->nextsel;
      if (unode->next == delnode) unode->next = delnode->next;
    }
    if (root->nextsel == delnode) root->nextsel = delnode->next;
    if (root->next == delnode) root->next = delnode->next;
    if (!root->nextsel) uniselect(root, select_all);
    freeuninode(delnode); delnode = NULL;
    if (root->cnt > 0) root->cnt--;
    countsel(root);
    doublechain(root);
  }
  chkul(root, "remuninode end");
  CE;
}

/*
** moveuniroot vertauscht den inhalt von zwei knoten
*/
void moveuniroot(struct unilist *root, struct unilist *newroot)
{
  struct unilist *bnode = getuninode();
  
  CF(moveuniroot);
  if (!root || !newroot) {CE; return;}

  bnode->next = newroot->next;
  bnode->nextsel = newroot->nextsel;
  bnode->cnt = newroot->cnt;
  bnode->res = newroot->res;
  bnode->day = newroot->day;
  bnode->month = newroot->month;
  bnode->year = newroot->year;
  bnode->name = newroot->name;
  bnode->full = newroot->full;
  bnode->desc = newroot->desc;

  newroot->next = root->next;
  newroot->nextsel = root->nextsel;
  newroot->cnt = root->cnt;
  newroot->res = root->res;
  newroot->day = root->day;
  newroot->month = root->month;
  newroot->year = root->year;
  newroot->name = root->name;
  newroot->full = root->full;
  newroot->desc = root->desc;

  root->next = bnode->next;
  root->nextsel = bnode->nextsel;
  root->cnt = bnode->cnt;
  root->res = bnode->res;
  root->day = bnode->day;
  root->month = bnode->month;
  root->year = bnode->year;
  root->name = bnode->name;
  root->full = bnode->full;
  root->desc = bnode->desc;

  /* nicht freeuninode, weil sonst die strings von root geloescht werden */
  free(bnode); bnode = NULL;
  CE;
}

/* 
** movenodes entfernt alle knoten fuer die select eins zurueckgibt 
** aus der liste or und haengt sie an die liste nr an
*/
int movenodes(struct unilist *or, struct unilist *nr, 
	      int (*select)(const struct unilist *, const struct unilist *))
{
  struct unilist *op, *np;
  chkul(or, "movenode ora");
  chkul(nr, "movenode nra");

  CF(movenodes);

  op = or;
  np = nr;
  
  while (np->next) np = np->next;

  while (or->nextsel) 
    if (!select(or->nextsel, or)) or->nextsel = or->nextsel->nextsel;
    else {
      for (op = or; op->next != or->nextsel; op = op->next);
      
      np->next = or->nextsel;
      op->next = or->nextsel->next;
      or->nextsel = np->next->nextsel;
      np->next->nextsel = (nr->nextsel) ? np : NULL;
      if (!nr->nextsel) nr->nextsel = nr->next;
      np = np->next;
      np->next = NULL;
      nr->cnt++;
      or->cnt--;
    }
  uniselect(or, select_all);
  CE;
  chkul(or, "movenode ore");
  chkul(nr, "movenode nre");

  return nr->cnt;
}

/*
** countsel zaehlt die ueber die zeiger nextsel ausgewaehlten knoten
*/
void countsel(struct unilist *root)
{
  struct unilist *unode;
  int i = 0;

  CF(countsel);
  if (!root) {CE; return;}
  chkul(root, "countsel");
  if (!root->next) {
    root->cnt = 0;
    root->res = 0;
    root->day = 0;
    CE;
    return;
  }

  for (unode = root, root->res = 0; ((unode = unode->nextsel)); root->res++);

  if (root->nextsel)
    for (unode = root, root->day = i = 0; ((unode = unode->next));) {
       i++;
       if (root->nextsel == unode) {
	 root->day = i;
	 break;
       }
    }
  CE;
  chkul(root, "countsel end");
}

/*
** findsel findet den knoten, dessen daten im wurzelknoten der liste
** stehen. erstens ueber den zeiger nextsel, zweitens ueber stringcomp
** mit root->name und drittens den ersten knoten der liste
*/
void findsel(struct unilist *root)
{
  struct unilist *unode;
  int i = 0;

  CF(findsel);
  if (!root) {CE; return;}
  if (!root->next) {CE; return;}
  chkul(root, "findsel");


  /* zeiger existiert noch */
  if (root->nextsel) {
    if (root->name != root->nextsel->name) {
      free(root->name); root->name = NULL;
      if (root->nextsel->name) root->name = strdup(root->nextsel->name);
      else root->name = strdup("ERROR!");
    }
    if (root->full != root->nextsel->full) {
      free(root->full); root->full = NULL;
      if (root->nextsel->full) root->full = strdup(root->nextsel->full);
    }
    if (root->desc != root->nextsel->desc) {
      free(root->desc); root->desc = NULL;
      if (root->nextsel->desc) root->desc = strdup(root->nextsel->desc);
    }
    for (unode = root->nextsel, i = 1; ((unode = unode->nextsel)); i++);
    root->day = i;
    chkul(root, "findsel end");
    CE; return;
  }
  /* name ist noch vorhanden */
  if (root->name) {
    if (selselect(root)) {
      for (unode = root, i = root->day; (unode->next != NULL) && i-- > 0;
	   unode = unode->next);
      free(root->name); root->name = NULL;
      if (unode->name) root->name = strdup(unode->name);
      else root->name = strdup("ERROR!");
      free(root->full); root->full = NULL;
      if (unode->full) root->full = strdup(unode->full);
      free(root->desc); root->desc = NULL;
      if (unode->desc) root->desc = strdup(unode->desc);
      selselect(root);
    }
  }
  else {
    root->nextsel = root->next;
    free(root->name); root->name = NULL;
    if (root->next->name) root->name = strdup(root->next->name);
    else root->name = strdup("ERROR!");
    free(root->full); root->full = NULL;
    if (root->next->full) root->full = strdup(root->next->full);
    free(root->desc); root->desc = NULL;
    if (root->next->desc) root->desc = strdup(root->next->desc);
    root->day = 1;
  }
  CE;
  chkul(root, "findsel end2");
}  

/*
** findmaxeq findet den laengsten den ueber nextsel ausewaehlten knoten
** gemeinsamen string in ->name
*/
char *findmaxeq(struct unilist *f)
{
  static char s[BUFSIZ] = "";
  char *ps, *pe;
  
  CF(findmaxeq);
  if (!f) {CE; return NULL;}
  if (!(f->nextsel) || !(f->nextsel->name)) {CE; return NULL;}
    
  strcpy(s, f->nextsel->name);
  while ((f = f->nextsel)) {
    for (ps = s, pe = f->name; *ps++ == *pe++;)
      ;
    *--ps = '\0';
  }
  if (*s == '\0') {CE; return NULL;}
  CE;
  return s;
}

/*
** die pn_.... funktionen enthalten die formatanweisungen fuer die
** ausgaben. die funktionen entnehemen die daten dem uebergebenen
** knoten und geben die ausgabezeile formatiert in einem string 
** zurueck
*/

/*
** ausgabe von ->name in quotes
*/
char *pn_line(struct unilist *f)
{
  static char s[BUFSIZ];
  
  CF(pn_line);
  sprintf(s, " \"%s\"", f->name);
  CE;
  return s;
}

/*
** ausgabe von knoten der faxliste
*/
char *pn_fax(struct unilist *f)
{
  static char s[BUFSIZ];
  
  CF(pn_fax);
  sprintf(s, "  %2i %s %4i %c  %-20s %-40s\n", 
	  f->day,
	  monthname(f->month),
	  f->cnt,
	  f->res,
	  cutonend(f->name, 20),
	  cutonend(f->desc, 40));
  CE;
  return s;
}

/*
** ausgabe der namensliste
*/
char *pn_name(struct unilist *f)
{
  static char s[BUFSIZ];
  
  CF(pn_name);
  sprintf(s, "   %-40s %-30s\n", 
	  cutonend(f->name, 40),
	  cutonend(f->desc, 30));
  CE;
  return s;
}

/*
** ausgabe der file-liste
*/
char *pn_file(struct unilist *f)
{
  static char s[BUFSIZ];
  
  CF(pn_file);
  sprintf(s, "   %s\n", 
	  cutonend(f->name, 70));
  CE;
  return s;
}

/*
** ausgabe der send-liste
*/
char *pn_full(struct unilist *f)
{
  static char s[BUFSIZ];
  
  CF(pn_full);
  sprintf(s, "   %-70s\n", cutonbeg(f->full, 70));
  CE;
  return s;
}

/*
** funktion zur ausgabe der faxliste beim debuggen
*/
void testfax(struct unilist *f)
{
  CF(testfax);
  while ((f = f->next))
    printf("%s", pn_fax(f));
  CE;
}

/*
** ausgabe zum debuggen
*/
void printrootnode(struct unilist *f)
{
  CF(printrootnode);
  if (!f) {
    printf("-------- zeiger auf liste ist NULL --------");
    CE;
    return;
  }
  printf("\n\nroot: cnt: %i res: %i day: %i mon: %i year: %i next: %p nsel: %p\n",
	 f->cnt, f->res, f->day, f->month, f->year, f->next, f->nextsel);
  printf("name: %s\n", f->name);
  printf("full: %s\n", f->full);
  printf("desc: %s\n", f->desc);
  CE;
}

/*
** ausgabe der ueber nexsel ausgewaehlten knoten zum debuggen
*/
void testsel(struct unilist *f)
{
  CF(testsel);
  printrootnode(f);
  while ((f = f->nextsel))
    printf("%-30s%-30s%-30s %c %i %i %i\n", f->name, cutonbeg(f->full, 30), 
	   f->desc, f->res, f->day, f->month, f->year);
  CE;
}

/*
** ausgabe einer kompleten liste zum debugen
*/
void test(struct unilist *f)
{
  CF(test);
  printrootnode(f);
  while ((f = f->next))
    printf("%-30s%-30s%-30s %c %i %i %i\n", f->name, cutonbeg(f->full, 30), 
	   f->desc, f->res, f->day, f->month, f->year);
  CE;
}






